@@ -41,7 +41,7 @@
 #include <audacious/misc.h>
 #include <audacious/playlist.h>
 #include <audacious/plugin.h>
-#include <libaudcore/eventqueue.h>
+#include <libaudcore/hook.h>
 #include <libaudgui/libaudgui.h>
 #include <libaudgui/libaudgui-gtk.h>
 
@@ -122,7 +122,8 @@ static void cdaudio_error (const gchar * message_format, ...)
     msg = g_markup_vprintf_escaped (message_format, args);
     va_end (args);
 
-    event_queue_with_data_free ("interface show error", msg);
+    aud_interface_show_error (msg);
+    g_free (msg);
 }
 
 /* main thread only */
@@ -141,7 +142,7 @@ static void purge_playlist (gint playlist)
             length--;
         }
 
-        g_free (filename);
+        str_unref (filename);
     }
 }
 
@@ -518,16 +519,17 @@ static Tuple * make_tuple (const gchar * filename, VFSFile * file)
 
     if (!strcmp (filename, "cdda://"))
     {
-        gint i = 0;
-
         tuple = tuple_new_from_filename (filename);
-        tuple->nsubtunes = n_audio_tracks;
-        tuple->subtunes = g_malloc (sizeof *tuple->subtunes * tuple->nsubtunes);
+
+        gint subtunes[n_audio_tracks];
+        gint i = 0;
 
         /* only add the audio tracks to the playlist */
         for (trackno = firsttrackno; trackno <= lasttrackno; trackno++)
             if (cdda_track_audiop (pcdrom_drive, trackno))
-                tuple->subtunes[i++] = trackno;
+                subtunes[i ++] = trackno;
+
+        tuple_set_subtunes (tuple, n_audio_tracks, subtunes);
 
         goto DONE;
     }
@@ -551,23 +553,23 @@ static Tuple * make_tuple (const gchar * filename, VFSFile * file)
 
     if (strlen (trackinfo[trackno].performer))
     {
-        tuple_associate_string (tuple, FIELD_ARTIST, NULL,
+        tuple_set_str (tuple, FIELD_ARTIST, NULL,
                                     trackinfo[trackno].performer);
     }
     if (strlen (trackinfo[0].name))
     {
-        tuple_associate_string (tuple, FIELD_ALBUM, NULL,
+        tuple_set_str (tuple, FIELD_ALBUM, NULL,
                                     trackinfo[0].name);
     }
     if (strlen (trackinfo[trackno].name))
     {
-        tuple_associate_string (tuple, FIELD_TITLE, NULL,
+        tuple_set_str (tuple, FIELD_TITLE, NULL,
                                     trackinfo[trackno].name);
     }
 
-    tuple_associate_int (tuple, FIELD_TRACK_NUMBER, NULL, trackno);
+    tuple_set_int (tuple, FIELD_TRACK_NUMBER, NULL, trackno);
 
-    tuple_associate_int (tuple, FIELD_LENGTH, NULL,
+    tuple_set_int (tuple, FIELD_LENGTH, NULL,
                              calculate_track_length (trackinfo[trackno].
                                                      startlsn,
                                                      trackinfo[trackno].
@@ -575,7 +577,7 @@ static Tuple * make_tuple (const gchar * filename, VFSFile * file)
 
     if (strlen (trackinfo[trackno].genre))
     {
-        tuple_associate_string (tuple, FIELD_GENRE, NULL,
+        tuple_set_str (tuple, FIELD_GENRE, NULL,
                                     trackinfo[trackno].genre);
     }
 
