@@ -40,17 +40,19 @@
 #endif
 #include <libaudcore/audstrings.h>
 
-/***********************************************************************************
- * Plugin glue.                                                                    *
- ***********************************************************************************/
-
 static GMutex *ctrl_mutex = NULL;
 static GCond *ctrl_cond = NULL;
 static gint64 seek_value = -1;
 static gboolean stop_flag = FALSE;
 
 static GStaticMutex data_mutex = G_STATIC_MUTEX_INIT;
-static mowgli_patricia_t * extension_dict = NULL;
+static GHashTable * extension_dict = NULL;
+
+/* str_unref() may be a macro */
+static void str_unref_cb (void * str)
+{
+    str_unref (str);
+}
 
 static gint lockmgr (void * * mutexp, enum AVLockOp op)
 {
@@ -75,17 +77,12 @@ static gint lockmgr (void * * mutexp, enum AVLockOp op)
 
 static gboolean ffaudio_init (void)
 {
-    avcodec_init();
     av_register_all();
     av_lockmgr_register (lockmgr);
 
     ctrl_mutex = g_mutex_new();
     ctrl_cond = g_cond_new();
 
-#ifdef FFAUDIO_USE_AUDTAG
-    tag_init();
-#endif
-
     return TRUE;
 }
 
@@ -97,7 +94,7 @@ ffaudio_cleanup(void)
     g_cond_free(ctrl_cond);
 
     if (extension_dict)
-        mowgli_patricia_destroy (extension_dict, NULL, NULL);
+        g_hash_table_destroy (extension_dict);
 
     av_lockmgr_register (NULL);
 }
@@ -108,9 +105,10 @@ static const gchar * ffaudio_strerror (gint error)
     return (! av_strerror (error, buf, sizeof buf)) ? buf : "unknown error";
 }
 
-static mowgli_patricia_t * create_extension_dict (void)
+static GHashTable * create_extension_dict (void)
 {
-    mowgli_patricia_t * dict = mowgli_patricia_create (NULL);
+    GHashTable * dict = g_hash_table_new_full (g_str_hash, g_str_equal,
+     str_unref_cb, NULL);
 
     AVInputFormat * f;
     for (f = av_iformat_next (NULL); f; f = av_iformat_next (f))
@@ -130,7 +128,7 @@ static mowgli_patricia_t * create_extension_dict (void)
                 next ++;
             }
 
-            mowgli_patricia_add (dict, parse, f);
+            g_hash_table_insert (dict, str_get(parse), f);
         }
 
         g_free (exts);
@@ -141,17 +139,21 @@ static mowgli_patricia_t * create_extension_dict (void)
 
 static AVInputFormat * get_format_by_extension (const gchar * name)
 {
-    gchar * ext = uri_get_extension (name);
-    if (! ext)
+    const gchar * ext0, * sub;
+    uri_parse (name, NULL, & ext0, & sub, NULL);
+
+    if (ext0 == sub)
         return NULL;
 
+    gchar * ext = g_ascii_strdown (ext0 + 1, sub - ext0 - 1);
+
     AUDDBG ("Get format by extension: %s\n", name);
     g_static_mutex_lock (& data_mutex);
 
     if (! extension_dict)
         extension_dict = create_extension_dict ();
 
-    AVInputFormat * f = mowgli_patricia_retrieve (extension_dict, ext);
+    AVInputFormat * f = g_hash_table_lookup (extension_dict, ext);
     g_static_mutex_unlock (& data_mutex);
 
     if (f)
@@ -217,7 +219,7 @@ static AVInputFormat * get_format (const gchar * name, VFSFile * file)
 
 static AVFormatContext * open_input_file (const gchar * name, VFSFile * file)
 {
-    AVInputFormat * f = get_format (file->uri, file);
+    AVInputFormat * f = get_format (name, file);
 
     if (! f)
     {
@@ -244,7 +246,13 @@ static AVFormatContext * open_input_file (const gchar * name, VFSFile * file)
 static void close_input_file (AVFormatContext * c)
 {
     AVIOContext * io = c->pb;
+
+#if CHECK_LIBAVFORMAT_VERSION (53, 24, 1)
+    avformat_close_input (&c);
+#else
     av_close_input_file (c);
+#endif
+
     io_context_free (io);
 }
 
@@ -310,11 +318,11 @@ ffaudio_get_meta(Tuple *tuple, AVFormatContext *ic, const ffaudio_meta_t *m)
     {
         switch (m->ttype) {
         case TUPLE_STRING:
-            tuple_associate_string (tuple, m->field, NULL, tag->value);
+            tuple_set_str (tuple, m->field, NULL, tag->value);
             break;
 
         case TUPLE_INT:
-            tuple_associate_int (tuple, m->field, NULL, atoi(tag->value));
+            tuple_set_int (tuple, m->field, NULL, atoi(tag->value));
             break;
 
         default:
@@ -332,13 +340,13 @@ ffaudio_get_tuple_data(Tuple *tuple, AVFormatContext *ic, AVCodecContext *c, AVC
         for (i = 0; i < n_metaentries; i++)
             ffaudio_get_meta(tuple, ic, &metaentries[i]);
 
-        tuple_associate_int(tuple, FIELD_LENGTH, NULL, ic->duration / 1000);
-        tuple_associate_int(tuple, FIELD_BITRATE, NULL, ic->bit_rate / 1000);
+        tuple_set_int(tuple, FIELD_LENGTH, NULL, ic->duration / 1000);
+        tuple_set_int(tuple, FIELD_BITRATE, NULL, ic->bit_rate / 1000);
     }
 
     if (codec != NULL && codec->long_name != NULL)
     {
-        tuple_associate_string(tuple, FIELD_CODEC, NULL, codec->long_name);
+        tuple_set_str(tuple, FIELD_CODEC, NULL, codec->long_name);
     }
 }
 
@@ -359,7 +367,7 @@ static Tuple * read_tuple (const gchar * filename, VFSFile * file)
         c = s->codec;
         if (c->codec_type == AVMEDIA_TYPE_AUDIO)
         {
-            av_find_stream_info(ic);
+            avformat_find_stream_info (ic, NULL);
             codec = avcodec_find_decoder(c->codec_id);
             if (codec != NULL)
                 break;
@@ -397,14 +405,8 @@ static gboolean ffaudio_write_tag (const Tuple * tuple, VFSFile * file)
     if (! file)
         return FALSE;
 
-    gchar *file_uri = g_ascii_strdown(file->uri, -4);
-
-    if (g_str_has_suffix(file_uri, ".ape"))
-    {
-        g_free(file_uri);
+    if (str_has_suffix_nocase (vfs_get_filename (file), ".ape"))
         return tag_tuple_write(tuple, file, TAG_TYPE_APE);
-    }
-    g_free(file_uri);
 
     return tag_tuple_write(tuple, file, TAG_TYPE_NONE);
 }
@@ -421,11 +423,8 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
     AVCodecContext *c = NULL;
     AVStream *s = NULL;
     AVPacket pkt = {.data = NULL};
-    guint8 *outbuf = NULL, *resbuf = NULL;
     gint i, stream_id, errcount;
-    gint in_sample_size, out_sample_size, chunk_size;
-    ReSampleContext *resctx = NULL;
-    gboolean codec_opened = FALSE, do_resampling = FALSE;
+    gboolean codec_opened = FALSE;
     gint out_fmt;
     gboolean seekable;
     gboolean error = FALSE;
@@ -440,7 +439,7 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
         c = s->codec;
         if (c->codec_type == AVMEDIA_TYPE_AUDIO)
         {
-            av_find_stream_info(ic);
+            avformat_find_stream_info (ic, NULL);
             codec = avcodec_find_decoder(c->codec_id);
             stream_id = i;
             if (codec != NULL)
@@ -456,42 +455,19 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
 
     AUDDBG("got codec %s for stream index %d, opening\n", codec->name, stream_id);
 
-    if (avcodec_open(c, codec) < 0)
+    if (avcodec_open2 (c, codec, NULL) < 0)
         goto error_exit;
 
     codec_opened = TRUE;
 
-    /* Determine if audio conversion or resampling is needed */
-    in_sample_size = av_get_bytes_per_sample (c->sample_fmt);
-    out_sample_size = av_get_bytes_per_sample (SAMPLE_FMT_S16);
-
-    chunk_size = out_sample_size * c->channels * (c->sample_rate / 50);
-
     switch (c->sample_fmt) {
         case SAMPLE_FMT_U8: out_fmt = FMT_U8; break;
         case SAMPLE_FMT_S16: out_fmt = FMT_S16_NE; break;
         case SAMPLE_FMT_S32: out_fmt = FMT_S32_NE; break;
         case SAMPLE_FMT_FLT: out_fmt = FMT_FLOAT; break;
-        default: do_resampling = TRUE; break;
-    }
-
-    if (do_resampling)
-    {
-        /* Initialize resampling context */
-        out_fmt = FMT_S16_NE;
-
-        AUDDBG("resampling needed chn=%d, rate=%d, fmt=%d -> chn=%d, rate=%d, fmt=S16NE\n",
-            c->channels, c->sample_rate, c->sample_fmt,
-            c->channels, c->sample_rate);
-
-        resctx = av_audio_resample_init(
-            c->channels, c->channels,
-            c->sample_rate, c->sample_rate,
-            SAMPLE_FMT_S16, c->sample_fmt,
-            16, 10, 0, 0.8);
-
-        if (resctx == NULL)
-            goto error_exit;
+    default:
+        fprintf (stderr, "ffaudio: Unsupported audio format %d\n", (int) c->sample_fmt);
+        goto error_exit;
     }
 
     /* Open audio output */
@@ -505,10 +481,6 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
 
     playback->set_gain_from_playlist(playback);
 
-    /* Allocate output buffer aligned to 16 byte boundary */
-    outbuf = av_malloc(AVCODEC_MAX_AUDIO_FRAME_SIZE);
-    resbuf = av_malloc(AVCODEC_MAX_AUDIO_FRAME_SIZE);
-
     AUDDBG("setting parameters\n");
 
     if (pause)
@@ -581,9 +553,6 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
         memcpy(&tmp, &pkt, sizeof(tmp));
         while (tmp.size > 0 && !stop_flag)
         {
-            gint len, out_size = AVCODEC_MAX_AUDIO_FRAME_SIZE;
-            guint8 *outbuf_p;
-
             /* Check for seek request and bail out if we have one */
             g_mutex_lock(ctrl_mutex);
             if (seek_value != -1)
@@ -601,59 +570,25 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
             }
             g_mutex_unlock(ctrl_mutex);
 
-            /* Decode whatever we can of the frame data */
-            len = avcodec_decode_audio3(c, (gint16 *)outbuf, &out_size, &tmp);
+            AVFrame * frame = avcodec_alloc_frame ();
+            int decoded = 0;
+            int len = avcodec_decode_audio4 (c, frame, & decoded, & tmp);
+
             if (len < 0)
             {
-                AUDDBG("codec failure, breaking out of loop\n");
+                fprintf (stderr, "ffaudio: decode_audio() failed, code %d\n", len);
                 break;
             }
 
             tmp.size -= len;
             tmp.data += len;
 
-            if (out_size <= 0)
+            if (! decoded)
                 continue;
 
-            /* Perform audio resampling if necessary */
-            if (do_resampling)
-            {
-                out_size = audio_resample(resctx,
-                    (gint16 *)resbuf, (gint16 *)outbuf,
-                    out_size / in_sample_size) * out_sample_size;
-                outbuf_p = resbuf;
-            }
-            else
-                outbuf_p = outbuf;
-
-            /* Output audio in small blocks */
-            while (out_size > 0 && !stop_flag && (stop_time < 0 ||
-             playback->output->written_time () < stop_time))
-            {
-                gint writeoff = MIN (chunk_size, out_size);
-
-                playback->output->write_audio((gint16 *)outbuf_p, writeoff);
-
-                outbuf_p += writeoff;
-                out_size -= writeoff;
-
-                /* Check for seek request and bail out if we have one */
-                g_mutex_lock(ctrl_mutex);
-                if (seek_value != -1)
-                {
-                    if (!seekable)
-                    {
-                        seek_value = -1;
-                        g_cond_signal(ctrl_cond);
-                    }
-                    else
-                    {
-                        g_mutex_unlock(ctrl_mutex);
-                        break;
-                    }
-                }
-                g_mutex_unlock(ctrl_mutex);
-            }
+            playback->output->write_audio (frame->data[0], FMT_SIZEOF (out_fmt)
+             * c->channels * frame->nb_samples);
+            av_free (frame);
         }
 
         if (pkt.data)
@@ -676,10 +611,6 @@ error_exit:
 
     stop_flag = TRUE;
 
-    av_free(outbuf);
-    av_free(resbuf);
-    if (resctx != NULL)
-        audio_resample_close(resctx);
     if (pkt.data)
         av_free_packet(&pkt);
     if (codec_opened)
