@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "plugins.h"
@@ -25,30 +25,28 @@
 
 #include <vorbis/vorbisenc.h>
 #include <stdlib.h>
+#include <stdint.h>
 
-static void vorbis_init(void);
+#include <audacious/gtk-compat.h>
+#include <audacious/misc.h>
+
+static void vorbis_init(write_output_callback write_output_func);
 static void vorbis_configure(void);
 static gint vorbis_open(void);
 static void vorbis_write(gpointer data, gint length);
 static void vorbis_close(void);
-static gint vorbis_free(void);
-static gint vorbis_playing(void);
-static gint vorbis_get_written_time(void);
+static gint (*write_output)(void *ptr, gint length);
 
 FileWriter vorbis_plugin =
 {
-    vorbis_init,
-    vorbis_configure,
-    vorbis_open,
-    vorbis_write,
-    vorbis_close,
-    vorbis_free,
-    vorbis_playing,
-    vorbis_get_written_time
+    .init = vorbis_init,
+    .configure = vorbis_configure,
+    .open = vorbis_open,
+    .write = vorbis_write,
+    .close = vorbis_close,
+    .format_required = FMT_FLOAT,
 };
 
-static float v_base_quality = 0.5;
-
 static ogg_stream_state os;
 static ogg_page og;
 static ogg_packet op;
@@ -58,66 +56,71 @@ static vorbis_block vb;
 static vorbis_info vi;
 static vorbis_comment vc;
 
-static float **encbuffer;
-static guint64 olen = 0;
+static const gchar * const vorbis_defaults[] = {
+ "base_quality", "0.5",
+ NULL};
 
-static void vorbis_init(void)
+static gdouble v_base_quality;
+
+static void vorbis_init(write_output_callback write_output_func)
 {
-    mcs_handle_t *db = aud_cfg_db_open();
+    aud_config_set_defaults ("filewriter_vorbis", vorbis_defaults);
 
-    aud_cfg_db_get_float(db, "filewriter_vorbis", "base_quality", &v_base_quality);
+    v_base_quality = aud_get_double ("filewriter_vorbis", "base_quality");
 
-    aud_cfg_db_close(db);
+    if (write_output_func)
+        write_output=write_output_func;
+}
+
+static void add_string_from_tuple (vorbis_comment * vc, const char * name,
+ const Tuple * tuple, gint field)
+{
+    gchar * val = tuple_get_str (tuple, field, NULL);
+    if (! val)
+        return;
+
+    vorbis_comment_add_tag (vc, name, val);
+    str_unref (val);
 }
 
 static gint vorbis_open(void)
 {
-    gint result;
     ogg_packet header;
     ogg_packet header_comm;
     ogg_packet header_code;
 
-    vorbis_init();
-
-    written = 0;
-    olen = 0;
+    vorbis_init(NULL);
 
     vorbis_info_init(&vi);
     vorbis_comment_init(&vc);
 
     if (tuple)
     {
-        const gchar *scratch;
         gchar tmpstr[32];
         gint scrint;
 
-        if ((scratch = aud_tuple_get_string(tuple, FIELD_TITLE, NULL)))
-            vorbis_comment_add_tag(&vc, "title", (gchar *) scratch);
-        if ((scratch = aud_tuple_get_string(tuple, FIELD_ARTIST, NULL)))
-            vorbis_comment_add_tag(&vc, "artist", (gchar *) scratch);
-        if ((scratch = aud_tuple_get_string(tuple, FIELD_ALBUM, NULL)))
-            vorbis_comment_add_tag(&vc, "album", (gchar *) scratch);
-        if ((scratch = aud_tuple_get_string(tuple, FIELD_GENRE, NULL)))
-            vorbis_comment_add_tag(&vc, "genre", (gchar *) scratch);
-        if ((scratch = aud_tuple_get_string(tuple, FIELD_DATE, NULL)))
-            vorbis_comment_add_tag(&vc, "date", (gchar *) scratch);
-        if ((scratch = aud_tuple_get_string(tuple, FIELD_COMMENT, NULL)))
-            vorbis_comment_add_tag(&vc, "comment", (gchar *) scratch);
+        add_string_from_tuple (& vc, "title", tuple, FIELD_TITLE);
+        add_string_from_tuple (& vc, "artist", tuple, FIELD_ARTIST);
+        add_string_from_tuple (& vc, "album", tuple, FIELD_ALBUM);
+        add_string_from_tuple (& vc, "genre", tuple, FIELD_GENRE);
+        add_string_from_tuple (& vc, "date", tuple, FIELD_DATE);
+        add_string_from_tuple (& vc, "comment", tuple, FIELD_COMMENT);
 
-        if ((scrint = aud_tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL)))
+        if ((scrint = tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL)))
         {
             g_snprintf(tmpstr, sizeof(tmpstr), "%d", scrint);
             vorbis_comment_add_tag(&vc, "tracknumber", tmpstr);
         }
 
-        if ((scrint = aud_tuple_get_int(tuple, FIELD_YEAR, NULL)))
+        if ((scrint = tuple_get_int(tuple, FIELD_YEAR, NULL)))
         {
             g_snprintf(tmpstr, sizeof(tmpstr), "%d", scrint);
             vorbis_comment_add_tag(&vc, "year", tmpstr);
         }
     }
 
-    if ((result = vorbis_encode_init_vbr(&vi, (long)input.channels, (long)input.frequency, v_base_quality)) != 0)
+    if (vorbis_encode_init_vbr (& vi, input.channels, input.frequency,
+     v_base_quality))
     {
         vorbis_info_clear(&vi);
         return 0;
@@ -135,50 +138,32 @@ static gint vorbis_open(void)
     ogg_stream_packetin(&os, &header_comm);
     ogg_stream_packetin(&os, &header_code);
 
-    while((result = ogg_stream_flush(&os, &og)))
+    while (ogg_stream_flush (& os, & og))
     {
-        if (result == 0)
-            break;
-
-        written += aud_vfs_fwrite(og.header, 1, og.header_len, output_file);
-        written += aud_vfs_fwrite(og.body, 1, og.body_len, output_file);
+        write_output(og.header, og.header_len);
+        write_output(og.body, og.body_len);
     }
 
     return 1;
 }
 
-static void vorbis_write(gpointer data, gint length)
+static void vorbis_write_real (void * data, gint length)
 {
-    int i;
-    int result;
-    short int *tmpdata;
+    int samples = length / sizeof (float);
+    int channel, result;
+    float * end = (float *) data + samples;
+    float * * buffer = vorbis_analysis_buffer (& vd, samples / input.channels);
+    float * from, * to;
 
-    /* ask vorbisenc for a buffer to fill with pcm data */
-    encbuffer = vorbis_analysis_buffer(&vd, length);
-    tmpdata = data;
+    for (channel = 0; channel < input.channels; channel ++)
+    {
+        to = buffer[channel];
 
-    /*
-     * deinterleave the audio signal, 32768.0 is the highest peak level allowed
-     * in a 16-bit PCM signal.
-     */
-    if (input.channels == 1)
-    {
-        for (i = 0; i < (length / 2); i++)
-        {
-            encbuffer[0][i] = tmpdata[i] / 32768.0;
-            encbuffer[1][i] = tmpdata[i] / 32768.0;
-        }
-    }
-    else
-    {
-        for (i = 0; i < (length / 4); i++)
-        {
-            encbuffer[0][i] = tmpdata[2 * i] / 32768.0;
-            encbuffer[1][i] = tmpdata[2 * i + 1] / 32768.0;
-        }
+        for (from = (float *) data + channel; from < end; from += input.channels)
+            * to ++ = * from;
     }
 
-    vorbis_analysis_wrote(&vd, i);
+    vorbis_analysis_wrote (& vd, samples / input.channels);
 
     while(vorbis_analysis_blockout(&vd, &vb) == 1)
     {
@@ -194,17 +179,29 @@ static void vorbis_write(gpointer data, gint length)
                 if (result == 0)
                     break;
 
-                written += aud_vfs_fwrite(og.header, 1, og.header_len, output_file);
-                written += aud_vfs_fwrite(og.body, 1, og.body_len, output_file);
+                write_output(og.header, og.header_len);
+                write_output(og.body, og.body_len);
             }
         }
     }
+}
 
-    olen += length;
+static void vorbis_write (void * data, gint length)
+{
+    if (length > 0) /* don't signal end of file yet */
+        vorbis_write_real (data, length);
 }
 
 static void vorbis_close(void)
 {
+    vorbis_write_real (NULL, 0); /* signal end of file */
+
+    while (ogg_stream_flush (& os, & og))
+    {
+        write_output (og.header, og.header_len);
+        write_output (og.body, og.body_len);
+    }
+
     ogg_stream_clear(&os);
 
     vorbis_block_clear(&vb);
@@ -212,44 +209,19 @@ static void vorbis_close(void)
     vorbis_info_clear(&vi);
 }
 
-static gint vorbis_free(void)
-{
-    return 1000000;
-}
-
-static gint vorbis_playing(void)
-{
-    return 0;
-}
-
-static gint vorbis_get_written_time(void)
-{
-    if (input.frequency && input.channels)
-        return (gint) ((olen * 1000) / (input.frequency * 2 * input.channels) + offset);
-
-    return 0;
-}
-
 /* configuration stuff */
 static GtkWidget *configure_win = NULL;
 static GtkWidget *quality_frame, *quality_vbox, *quality_hbox1, *quality_spin, *quality_label;
-static GtkObject *quality_adj;
+static GtkAdjustment * quality_adj;
 
 static void quality_change(GtkAdjustment *adjustment, gpointer user_data)
 {
-    if (gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(quality_spin)))
-        v_base_quality = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(quality_spin)) / 10;
-    else
-        v_base_quality = 0.0;
+    v_base_quality = gtk_spin_button_get_value ((GtkSpinButton *) quality_spin) / 10;
 }
 
 static void configure_ok_cb(gpointer data)
 {
-    mcs_handle_t *db = aud_cfg_db_open();
-
-    aud_cfg_db_set_float(db, "filewrite_vorbis", "base_quality", v_base_quality);
-
-    aud_cfg_db_close(db);
+    aud_set_double ("filewrite_vorbis", "base_quality", v_base_quality);
 
     gtk_widget_hide(configure_win);
 }
@@ -262,6 +234,7 @@ static void vorbis_configure(void)
     if (configure_win == NULL)
     {
         configure_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+        gtk_window_set_type_hint(GTK_WINDOW(configure_win), GDK_WINDOW_TYPE_HINT_DIALOG);
         g_signal_connect(G_OBJECT(configure_win), "destroy", G_CALLBACK(gtk_widget_destroyed), NULL);
 
         gtk_window_set_title(GTK_WINDOW(configure_win), _("Vorbis Encoder Configuration"));
@@ -288,7 +261,7 @@ static void vorbis_configure(void)
         gtk_misc_set_alignment(GTK_MISC(quality_label), 0, 0.5);
         gtk_box_pack_start(GTK_BOX(quality_hbox1), quality_label, TRUE, TRUE, 0);
 
-        quality_adj = gtk_adjustment_new(5, 0, 10, 0.1, 1, 1);
+        quality_adj = (GtkAdjustment *) gtk_adjustment_new (5, 0, 10, 0.1, 1, 0);
         quality_spin = gtk_spin_button_new(GTK_ADJUSTMENT(quality_adj), 1, 2);
         gtk_box_pack_start(GTK_BOX(quality_hbox1), quality_spin, TRUE, TRUE, 0);
         g_signal_connect(G_OBJECT(quality_adj), "value-changed", G_CALLBACK(quality_change), NULL);
@@ -298,17 +271,16 @@ static void vorbis_configure(void)
         /* buttons */
         bbox = gtk_hbutton_box_new();
         gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
-        gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
         gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
         button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-        g_signal_connect_swapped(G_OBJECT(button), "clicked", G_CALLBACK(gtk_widget_hide), GTK_OBJECT(configure_win));
+        g_signal_connect_swapped (button, "clicked", (GCallback)
+         gtk_widget_hide, configure_win);
         gtk_box_pack_start(GTK_BOX(bbox), button, TRUE, TRUE, 0);
 
         button = gtk_button_new_from_stock(GTK_STOCK_OK);
         g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(configure_ok_cb), NULL);
         gtk_box_pack_start(GTK_BOX(bbox), button, TRUE, TRUE, 0);
-        gtk_widget_grab_default(button);
     }
 
     gtk_widget_show_all(configure_win);
