@@ -26,6 +26,7 @@
 #include <audacious/i18n.h>
 #include <audacious/playlist.h>
 #include <audacious/plugin.h>
+#include <audacious/plugins.h>
 #include <audacious/misc.h>
 #include <libaudcore/hook.h>
 #include <libaudgui/libaudgui.h>
@@ -47,13 +48,13 @@ static const gchar * const gtkui_defaults[] = {
  "statusbar_visible", "TRUE",
 
  "autoscroll", "TRUE",
- "playlist_columns", "number title artist album queued length",
+ "playlist_columns", "title artist album queued length",
  "playlist_headers", "TRUE",
 
  "player_x", "-1",
  "player_y", "-1",
- "player_width", "600",
- "player_height", "400",
+ "player_width", "760",
+ "player_height", "460",
 
  /* hidden settings */
  "always_on_top", "FALSE",
@@ -62,6 +63,8 @@ static const gchar * const gtkui_defaults[] = {
  "custom_playlist_colors", "FALSE",
  NULL};
 
+static PluginHandle * search_tool;
+
 static GtkWidget *volume;
 static gboolean volume_slider_is_moving = FALSE;
 static guint update_volume_timeout_source = 0;
@@ -70,9 +73,8 @@ static gulong volume_change_handler_id;
 static GtkAccelGroup * accel;
 
 static GtkWidget * button_play, * button_pause, * button_stop, * slider,
- * label_time, * button_shuffle, * button_repeat;
-static GtkWidget * window, * vbox, * menu_box, * menu, * playlist_box,
- * infoarea, * statusbar;
+ * label_time, * button_shuffle, * button_repeat, * search_button;
+static GtkWidget * window, * vbox_outer, * vbox, * menu_box, * menu, * infoarea, * statusbar;
 static GtkWidget * menu_main, * menu_rclick, * menu_tab;
 
 static GtkWidget * error_win = NULL;
@@ -81,8 +83,6 @@ static gboolean slider_is_moving = FALSE;
 static guint delayed_title_change_source = 0;
 static guint update_song_timeout_source = 0;
 
-extern GtkWidget *ui_playlist_notebook_tab_title_editing;
-
 static gboolean init (void);
 static void cleanup (void);
 static void ui_show (gboolean show);
@@ -156,7 +156,7 @@ static gboolean title_change_cb (void)
             gchar * title_s = g_strdup_printf (_("%s - Audacious"), title);
             gtk_window_set_title ((GtkWindow *) window, title_s);
             g_free (title_s);
-            g_free (title);
+            str_unref (title);
         }
         else
             gtk_window_set_title ((GtkWindow *) window, _("Buffering ..."));
@@ -203,7 +203,12 @@ static gboolean ui_is_shown (void)
 
 static gboolean ui_is_focused (void)
 {
+/* gtk_window_is_active() is too unreliable, unfortunately. --jlindgren */
+#if 0
     return gtk_window_is_active ((GtkWindow *) window);
+#else
+    return ui_is_shown ();
+#endif
 }
 
 static void ui_show_error (const gchar * text)
@@ -420,7 +425,7 @@ static gboolean rclick_cb (GtkWidget * widget, GdkEventButton * event)
     return TRUE;
 }
 
-static GtkWidget *gtk_toolbar_button_add(GtkWidget * toolbar, void (*callback) (), const gchar * stock_id)
+static GtkWidget *toolbar_button_add(GtkWidget * toolbar, void (*callback) (), const gchar * stock_id)
 {
     GtkWidget *icon;
     GtkWidget *button = gtk_button_new();
@@ -445,7 +450,10 @@ static GtkWidget * toggle_button_new (const gchar * icon, const gchar * alt,
     gtk_widget_set_can_focus (button, FALSE);
     gtk_button_set_relief ((GtkButton *) button, GTK_RELIEF_NONE);
 
-    if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), icon))
+    if (! alt)
+        gtk_container_add ((GtkContainer *) button, gtk_image_new_from_stock
+         (icon, GTK_ICON_SIZE_BUTTON));
+    else if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), icon))
         gtk_container_add ((GtkContainer *) button, gtk_image_new_from_icon_name
          (icon, GTK_ICON_SIZE_BUTTON));
     else
@@ -461,77 +469,22 @@ static GtkWidget * toggle_button_new (const gchar * icon, const gchar * alt,
     return button;
 }
 
-static GtkWidget *gtk_markup_label_new(const gchar * str)
+static GtkWidget *markup_label_new(const gchar * str)
 {
     GtkWidget *label = gtk_label_new(str);
     g_object_set(G_OBJECT(label), "use-markup", TRUE, NULL);
     return label;
 }
 
-static gboolean ui_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+static void window_mapped_cb (GtkWidget * widget, void * unused)
 {
-    if (ui_playlist_notebook_tab_title_editing)
-    {
-        if (event->keyval == GDK_KP_Enter || event->keyval == GDK_Escape)
-            return FALSE;
-
-        GtkWidget *entry = g_object_get_data(G_OBJECT(ui_playlist_notebook_tab_title_editing), "entry");
-        gtk_widget_event(entry, (GdkEvent*) event);
-        return TRUE;
-    }
+    gtk_widget_grab_focus (playlist_get_treeview (aud_playlist_get_active ()));
+}
 
+static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, void * unused)
+{
     switch (event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK))
     {
-        case 0:
-            switch (event->keyval)
-            {
-                case GDK_Left:
-                case GDK_KP_Left:
-                case GDK_KP_7:
-                    if (aud_drct_get_playing ())
-                        aud_drct_seek (aud_drct_get_time () - 5000);
-                    break;
-
-                case GDK_Right:
-                case GDK_KP_Right:
-                case GDK_KP_9:
-                    if (aud_drct_get_playing ())
-                        aud_drct_seek (aud_drct_get_time () + 5000);
-                    break;
-
-                case GDK_KP_4:
-                    aud_drct_pl_prev();
-                    break;
-
-                case GDK_KP_6:
-                    aud_drct_pl_next();
-                    break;
-
-                case GDK_space:
-                    if (aud_drct_get_playing())
-                        aud_drct_pause();
-                    else
-                        aud_drct_play();
-                    break;
-
-                case GDK_Escape:
-                    ui_playlist_notebook_position (GINT_TO_POINTER
-                     (aud_playlist_get_active ()), NULL);
-                    break;
-
-                case GDK_Delete:
-                    playlist_delete_selected ();
-                    break;
-
-                case GDK_Menu:
-                    popup_menu_rclick (0, event->time);
-                    break;
-
-                default:
-                    return FALSE;
-            }
-            break;
-
       case GDK_CONTROL_MASK:
         switch (event->keyval)
         {
@@ -540,6 +493,7 @@ static gboolean ui_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
             aud_playlist_set_active ((aud_playlist_get_active () + 1) %
              aud_playlist_count ());
             break;
+
           default:
             return FALSE;
         }
@@ -559,11 +513,13 @@ static gboolean ui_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
       case GDK_MOD1_MASK:
         switch (event->keyval)
         {
-          case GDK_Up:
-            playlist_shift (-1);
+          case GDK_Left:
+            if (aud_drct_get_playing ())
+                aud_drct_seek (aud_drct_get_time () - 5000);
             break;
-          case GDK_Down:
-            playlist_shift (1);
+          case GDK_Right:
+            if (aud_drct_get_playing ())
+                aud_drct_seek (aud_drct_get_time () + 5000);
             break;
           default:
             return FALSE;
@@ -575,6 +531,48 @@ static gboolean ui_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
     return TRUE;
 }
 
+static gboolean playlist_keypress_cb (GtkWidget * widget, GdkEventKey * event, void * unused)
+{
+    switch (event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK))
+    {
+    case 0:
+        switch (event->keyval)
+        {
+        case GDK_Escape:
+            ui_playlist_notebook_position (GINT_TO_POINTER (aud_playlist_get_active ()), NULL);
+            return TRUE;
+        case GDK_Delete:
+            playlist_delete_selected ();
+            return TRUE;
+        case GDK_Menu:
+            popup_menu_rclick (0, event->time);
+            return TRUE;
+        }
+
+        break;
+    case GDK_CONTROL_MASK:
+        switch (event->keyval)
+        {
+        case 'x':
+            playlist_cut ();
+            return TRUE;
+        case 'c':
+            playlist_copy ();
+            return TRUE;
+        case 'v':
+            playlist_paste ();
+            return TRUE;
+        case 'a':
+            aud_playlist_select_all (aud_playlist_get_active (), TRUE);
+            return TRUE;
+        }
+
+        break;
+    }
+
+    return FALSE;
+}
+
 static void update_toggles (void * data, void * user)
 {
     gtk_toggle_button_set_active ((GtkToggleButton *) button_repeat, aud_get_bool (NULL, "repeat"));
@@ -591,6 +589,18 @@ static void toggle_shuffle (GtkToggleButton * button, void * unused)
     aud_set_bool (NULL, "shuffle", gtk_toggle_button_get_active (button));
 }
 
+static void toggle_search_tool (GtkToggleButton * button, void * unused)
+{
+    aud_plugin_enable (search_tool, gtk_toggle_button_get_active (button));
+}
+
+static gboolean search_tool_toggled (PluginHandle * plugin, void * unused)
+{
+    gtk_toggle_button_set_active ((GtkToggleButton *) search_button,
+     aud_plugin_get_enabled (plugin));
+    return TRUE;
+}
+
 static void config_save (void)
 {
     save_window_size ();
@@ -636,6 +646,8 @@ static void ui_hooks_disassociate(void)
 
 static gboolean init (void)
 {
+    search_tool = aud_plugin_lookup_basename ("search-tool");
+
     GtkWidget *tophbox;         /* box to contain toolbar and shbox */
     GtkWidget *buttonbox;       /* contains buttons like "open", "next" */
     GtkWidget *shbox;           /* box for volume control + slider + time combo --nenolod */
@@ -662,27 +674,37 @@ static gboolean init (void)
 
     g_signal_connect(G_OBJECT(window), "delete-event", G_CALLBACK(window_delete), NULL);
 
-    vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_add(GTK_CONTAINER(window), vbox);
-
     accel = gtk_accel_group_new ();
     gtk_window_add_accel_group ((GtkWindow *) window, accel);
 
+    vbox_outer = gtk_vbox_new (FALSE, 0);
+    gtk_container_add ((GtkContainer *) window, vbox_outer);
+
     menu_box = gtk_hbox_new (FALSE, 0);
-    gtk_box_pack_start ((GtkBox *) vbox, menu_box, FALSE, FALSE, 0);
+    gtk_box_pack_start ((GtkBox *) vbox_outer, menu_box, FALSE, FALSE, 0);
     show_menu (aud_get_bool ("gtkui", "menu_visible"));
 
     tophbox = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(vbox), tophbox, FALSE, TRUE, 0);
+    gtk_box_pack_start ((GtkBox *) vbox_outer, tophbox, FALSE, FALSE, 0);
 
     buttonbox = gtk_hbox_new(FALSE, 0);
-    gtk_toolbar_button_add(buttonbox, button_open_pressed, GTK_STOCK_OPEN);
-    gtk_toolbar_button_add(buttonbox, button_add_pressed, GTK_STOCK_ADD);
-    button_play = gtk_toolbar_button_add (buttonbox, aud_drct_play, GTK_STOCK_MEDIA_PLAY);
-    button_pause = gtk_toolbar_button_add (buttonbox, aud_drct_pause, GTK_STOCK_MEDIA_PAUSE);
-    button_stop = gtk_toolbar_button_add (buttonbox, aud_drct_stop, GTK_STOCK_MEDIA_STOP);
-    gtk_toolbar_button_add (buttonbox, aud_drct_pl_prev, GTK_STOCK_MEDIA_PREVIOUS);
-    gtk_toolbar_button_add (buttonbox, aud_drct_pl_next, GTK_STOCK_MEDIA_NEXT);
+
+    if (search_tool)
+    {
+        search_button = toggle_button_new (GTK_STOCK_FIND, NULL, toggle_search_tool, NULL);
+        gtk_box_pack_start ((GtkBox *) tophbox, search_button, FALSE, FALSE, 0);
+        gtk_toggle_button_set_active ((GtkToggleButton *) search_button,
+         aud_plugin_get_enabled (search_tool));
+        aud_plugin_add_watch (search_tool, search_tool_toggled, NULL);
+    }
+
+    toolbar_button_add(buttonbox, button_open_pressed, GTK_STOCK_OPEN);
+    toolbar_button_add(buttonbox, button_add_pressed, GTK_STOCK_ADD);
+    button_play = toolbar_button_add (buttonbox, aud_drct_play, GTK_STOCK_MEDIA_PLAY);
+    button_pause = toolbar_button_add (buttonbox, aud_drct_pause, GTK_STOCK_MEDIA_PAUSE);
+    button_stop = toolbar_button_add (buttonbox, aud_drct_stop, GTK_STOCK_MEDIA_STOP);
+    toolbar_button_add (buttonbox, aud_drct_pl_prev, GTK_STOCK_MEDIA_PREVIOUS);
+    toolbar_button_add (buttonbox, aud_drct_pl_next, GTK_STOCK_MEDIA_NEXT);
 
     /* Workaround: Show the play and pause buttons and then hide them again in
      * order to coax GTK into loading icons for them. -jlindgren */
@@ -711,7 +733,7 @@ static gboolean init (void)
     gtk_box_pack_start ((GtkBox *) shbox, slider, TRUE, TRUE, 6);
     gtk_widget_set_no_show_all (slider, TRUE);
 
-    label_time = gtk_markup_label_new(NULL);
+    label_time = markup_label_new(NULL);
     gtk_widget_set_no_show_all (label_time, TRUE);
 
     gtk_box_pack_end ((GtkBox *) shbox, label_time, FALSE, FALSE, 6);
@@ -720,8 +742,7 @@ static gboolean init (void)
     gtk_button_set_relief(GTK_BUTTON(volume), GTK_RELIEF_NONE);
     gtk_scale_button_set_adjustment(GTK_SCALE_BUTTON(volume), GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 100, 1, 5, 0)));
     gtk_widget_set_can_focus(volume, FALSE);
-    /* Set the default volume to the balance average.
-       (I'll add balance control later) -Ryan */
+
     gint lvol = 0, rvol = 0;
     aud_drct_get_volume(&lvol, &rvol);
     gtk_scale_button_set_value(GTK_SCALE_BUTTON(volume), (lvol + rvol) / 2);
@@ -735,24 +756,17 @@ static gboolean init (void)
      toggle_repeat, NULL);
     gtk_box_pack_end ((GtkBox *) tophbox, button_repeat, FALSE, FALSE, 0);
 
-    playlist_box = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(vbox), playlist_box, TRUE, TRUE, 0);
-
-    /* Create playlist notebook */
-    ui_playlist_notebook_new ();
-    g_object_ref (G_OBJECT(UI_PLAYLIST_NOTEBOOK));
-
-    if (aud_get_bool ("gtkui", "statusbar_visible"))
-    {
-        statusbar = ui_statusbar_new();
-        gtk_box_pack_end(GTK_BOX(vbox), statusbar, FALSE, FALSE, 3);
-    }
-
     layout_load ();
 
     GtkWidget * layout = layout_new ();
-    gtk_box_pack_start ((GtkBox *) playlist_box, layout, TRUE, TRUE, 0);
-    layout_add_center ((GtkWidget *) UI_PLAYLIST_NOTEBOOK);
+    gtk_box_pack_start ((GtkBox *) vbox_outer, layout, TRUE, TRUE, 0);
+
+    vbox = gtk_vbox_new (FALSE, 0);
+    layout_add_center (vbox);
+
+    /* Create playlist notebook */
+    ui_playlist_notebook_new ();
+    gtk_box_pack_start ((GtkBox *) vbox, (GtkWidget *) UI_PLAYLIST_NOTEBOOK, TRUE, TRUE, 0);
 
     if (aud_get_bool ("gtkui", "infoarea_visible"))
     {
@@ -760,6 +774,12 @@ static gboolean init (void)
         gtk_box_pack_end (GTK_BOX(vbox), infoarea, FALSE, FALSE, 0);
     }
 
+    if (aud_get_bool ("gtkui", "statusbar_visible"))
+    {
+        statusbar = ui_statusbar_new ();
+        gtk_box_pack_end ((GtkBox *) vbox_outer, statusbar, FALSE, FALSE, 0);
+    }
+
     AUDDBG("hooks associate\n");
     ui_hooks_associate();
 
@@ -775,7 +795,9 @@ static gboolean init (void)
     g_signal_connect(volume, "released", G_CALLBACK(ui_volume_released_cb), NULL);
     update_volume_timeout_source = g_timeout_add(250, (GSourceFunc) ui_volume_slider_update, volume);
 
-    g_signal_connect(window, "key-press-event", G_CALLBACK(ui_key_press_cb), NULL);
+    g_signal_connect (window, "map-event", (GCallback) window_mapped_cb, NULL);
+    g_signal_connect (window, "key-press-event", (GCallback) window_keypress_cb, NULL);
+    g_signal_connect (UI_PLAYLIST_NOTEBOOK, "key-press-event", (GCallback) playlist_keypress_cb, NULL);
 
     if (aud_drct_get_playing ())
     {
@@ -788,7 +810,7 @@ static gboolean init (void)
 
     title_change_cb ();
 
-    gtk_widget_show_all (vbox);
+    gtk_widget_show_all (vbox_outer);
 
     if (aud_get_bool ("gtkui", "player_visible"))
         ui_show (TRUE);
@@ -832,11 +854,11 @@ static void cleanup (void)
 
     ui_hooks_disassociate();
 
+    if (search_tool)
+        aud_plugin_remove_watch (search_tool, search_tool_toggled, NULL);
+
     pw_col_cleanup ();
-
-    g_object_unref ((GObject *) UI_PLAYLIST_NOTEBOOK);
     gtk_widget_destroy (window);
-
     layout_cleanup ();
 }
 
@@ -880,7 +902,6 @@ void show_infoarea (gboolean show)
     {
         infoarea = ui_infoarea_new ();
         gtk_box_pack_end ((GtkBox *) vbox, infoarea, FALSE, FALSE, 0);
-        gtk_box_reorder_child ((GtkBox *) vbox, infoarea, -1);
         gtk_widget_show_all (infoarea);
     }
 
@@ -898,11 +919,7 @@ void show_statusbar (gboolean show)
     if (show && ! statusbar)
     {
         statusbar = ui_statusbar_new ();
-        gtk_box_pack_end ((GtkBox *) vbox, statusbar, FALSE, FALSE, 3);
-
-        if (infoarea)
-            gtk_box_reorder_child ((GtkBox *) vbox, infoarea, -1);
-
+        gtk_box_pack_end ((GtkBox *) vbox_outer, statusbar, FALSE, FALSE, 0);
         gtk_widget_show_all (statusbar);
     }
 
