@@ -30,7 +30,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with audacious-hotkey; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -41,6 +41,9 @@
 
 #include <audacious/plugin.h>
 #include <audacious/i18n.h>
+#include <audacious/gtk-compat.h>
+#include <libaudgui/libaudgui.h>
+#include <libaudgui/libaudgui-gtk.h>
 
 #include "plugin.h"
 #include "gui.h"
@@ -77,7 +80,9 @@ static gchar* event_desc[EVENT_MAX] = {
 	[EVENT_VOL_DOWN] = N_("Volume Down"),
 	[EVENT_JUMP_TO_FILE] = N_("Jump to File"),
 	[EVENT_TOGGLE_WIN] = N_("Toggle Player Windows"),
-	[EVENT_SHOW_AOSD] = N_("Show On-Screen-Display")
+	[EVENT_SHOW_AOSD] = N_("Show On-Screen-Display"),
+	[EVENT_TOGGLE_REPEAT] = N_("Toggle Repeat"),
+	[EVENT_TOGGLE_SHUFFLE] = N_("Toggle Shuffle"),
 };
 
 
@@ -316,12 +321,12 @@ KeyControls* add_event_controls(KeyControls* list,
 		controls->hotkey.event = 0;
 	}
 
-	controls->combobox = gtk_combo_box_new_text();
+	controls->combobox = gtk_combo_box_text_new();
 	for (i=0;i<EVENT_MAX;i++)
 	{
-		gtk_combo_box_append_text( GTK_COMBO_BOX(controls->combobox), _(event_desc[i]) );
+		gtk_combo_box_text_append_text((GtkComboBoxText *) controls->combobox, _(event_desc[i]));
 	}
-	gtk_combo_box_set_active( GTK_COMBO_BOX(controls->combobox), controls->hotkey.event);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(controls->combobox), controls->hotkey.event);
 	gtk_table_attach (GTK_TABLE (table), controls->combobox, 0, 1, row, row+1,
 			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND), (GtkAttachOptions) (GTK_EXPAND), 0, 0);
 
@@ -329,7 +334,7 @@ KeyControls* add_event_controls(KeyControls* list,
 	controls->keytext = gtk_entry_new ();
 	gtk_table_attach (GTK_TABLE (table), controls->keytext, 1, 2, row, row+1,
 			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND), (GtkAttachOptions) (GTK_EXPAND), 0, 0);
-	gtk_entry_set_editable (GTK_ENTRY (controls->keytext), FALSE);
+	gtk_editable_set_editable(GTK_EDITABLE(controls->keytext), FALSE);
 
 
 	set_keytext(controls->keytext, controls->hotkey.key, controls->hotkey.mask, controls->hotkey.type);
@@ -650,25 +655,19 @@ void ok_callback (GtkWidget *widget, gpointer data)
 	gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (widget)));
 }
 
-
-
 void show_about (void)
 {
-	static GtkWidget *dialog;
+    static GtkWidget * dialog = NULL;
 
-	dialog = audacious_info_dialog (_("About Global Hotkey Plugin"),
-				_("Global Hotkey Plugin\n"
-				"Control the player with global key combinations or multimedia keys.\n\n"
-				"Copyright (C) 2007-2008 Sascha Hlusiak <contact@saschahlusiak.de>\n\n"
-				"Contributers include:\n"
-				"Copyright (C) 2006-2007 Vladimir Paskov <vlado.paskov@gmail.com>\n"
-				"Copyright (C) 2000-2002 Ville Syrjälä <syrjala@sci.fi>\n"
-                         	"			Bryn Davies <curious@ihug.com.au>\n"
-                        	"			Jonathan A. Davis <davis@jdhouse.org>\n"
-                         	"			Jeremy Tan <nsx@nsx.homeip.net>\n\n"
-                         	),
-                         	_("OK"), TRUE, NULL, NULL);
-
-	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
-			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
+    audgui_simple_message (& dialog, GTK_MESSAGE_INFO,
+     _("About Global Hotkey Plugin"),
+     _("Global Hotkey Plugin\n"
+     "Control the player with global key combinations or multimedia keys.\n\n"
+     "Copyright (C) 2007-2008 Sascha Hlusiak <contact@saschahlusiak.de>\n\n"
+     "Contributers include:\n"
+     "Copyright (C) 2006-2007 Vladimir Paskov <vlado.paskov@gmail.com>\n"
+     "Copyright (C) 2000-2002 Ville Syrjälä <syrjala@sci.fi>\n"
+     "			Bryn Davies <curious@ihug.com.au>\n"
+     "			Jonathan A. Davis <davis@jdhouse.org>\n"
+     "			Jeremy Tan <nsx@nsx.homeip.net>\n\n"));
 }
