@@ -1,23 +1,23 @@
 /*
- * OSS4 Output Plugin for Audacious
+ * oss.c
  * Copyright 2010-2011 Michał Lipski <tallica@o2.pl>
  *
  * I would like to thank people on #audacious, especially Tony Vroon and
  * John Lindgren and of course the authors of the previous OSS plugin.
  *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions, and the following disclaimer.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions, and the following disclaimer in the documentation
+ *    provided with the distribution.
  *
+ * This software is provided "as is" and without any warranty, express or
+ * implied. In no event shall the authors be liable for any damages arising from
+ * the use of this software.
  */
 
 #include "oss.h"
@@ -29,6 +29,7 @@ static const gchar * const oss_defaults[] = {
  "save_volume", "TRUE",
  "volume", "12850", /* 0x3232 */
  "cookedmode", "TRUE",
+ "exclusive", "FALSE",
  NULL};
 
 oss_data_t *oss_data;
@@ -94,15 +95,22 @@ FAILED:
 static gint open_device(void)
 {
     gint res = -1;
+    gint flags = O_WRONLY;
     gchar *device = aud_get_string("oss4", "device");
     gchar *alt_device = aud_get_string("oss4", "alt_device");
 
+    if (aud_get_bool("oss4", "exclusive"))
+    {
+        AUDDBG("Enabled exclusive mode.\n");
+        flags |= O_EXCL;
+    }
+
     if (aud_get_bool("oss4", "use_alt_device") && alt_device != NULL)
-        res = open(alt_device, O_WRONLY);
+        res = open(alt_device, flags);
     else if (device != NULL)
-        res = open(device, O_WRONLY);
+        res = open(device, flags);
     else
-        res = open(DEFAULT_DSP, O_WRONLY);
+        res = open(DEFAULT_DSP, flags);
 
     g_free(device);
     g_free(alt_device);
