@@ -27,10 +27,16 @@
 #include <limits.h>
 
 #include <gtk/gtk.h>
+#include <pulse/pulseaudio.h>
+
+#include <audacious/debug.h>
+#include <audacious/drct.h>
 #include <audacious/plugin.h>
 #include <audacious/i18n.h>
+#include <libaudgui/libaudgui.h>
+#include <libaudgui/libaudgui-gtk.h>
 
-#include <pulse/pulseaudio.h>
+#define ERROR(...) do {fprintf (stderr, "pulseaudio: " __VA_ARGS__); putchar ('\n');} while (0)
 
 static pa_context *context = NULL;
 static pa_stream *stream = NULL;
@@ -40,9 +46,9 @@ static pa_cvolume volume;
 static int volume_valid = 0;
 
 static int do_trigger = 0;
-static uint64_t written = 0;
-static int time_offset_msec = 0;
-static int just_flushed = 0;
+static int64_t written;
+static int flush_time;
+static int bytes_per_second;
 
 static int connected = 0;
 
@@ -53,7 +59,7 @@ if (!mainloop || \
     !context || pa_context_get_state(context) != PA_CONTEXT_READY || \
     !stream || pa_stream_get_state(stream) != PA_STREAM_READY) { \
         if (warn) \
-            g_warning("Connection died: %s", context ? pa_strerror(pa_context_errno(context)) : "NULL"); \
+            AUDDBG("Connection died: %s", context ? pa_strerror(pa_context_errno(context)) : "NULL"); \
         goto label; \
     }  \
 } while(0);
@@ -63,25 +69,6 @@ do { \
     if (!connected) return retval; \
 } while (0);
 
-/* This function is from xmms' core */
-gint ctrlsocket_get_session_id(void);
-
-static const char* get_song_name(void) {
-    static char t[256];
-    gint pos;
-    char *str, *u;
-    Playlist *playlist = aud_playlist_get_active();
-
-    pos = aud_playlist_get_position(playlist);
-    if (!(str = aud_playlist_get_songtitle(playlist, pos)))
-        return "Playback Stream";
-
-    snprintf(t, sizeof(t), "%s", u = pa_locale_to_utf8(str));
-    pa_xfree(u);
-
-    return t;
-}
-
 static void info_cb(struct pa_context *c, const struct pa_sink_input_info *i, int is_last, void *userdata) {
     assert(c);
 
@@ -104,7 +91,7 @@ static void subscribe_cb(struct pa_context *c, enum pa_subscription_event_type t
         return;
 
     if (!(o = pa_context_get_sink_input_info(c, index, info_cb, NULL))) {
-        g_warning("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(c)));
+        AUDDBG("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(c)));
         return;
     }
 
@@ -176,40 +163,34 @@ static void stream_latency_update_cb(pa_stream *s, void *userdata) {
     pa_threaded_mainloop_signal(mainloop, 0);
 }
 
-static void pulse_get_volume(int *l, int *r) {
-    pa_cvolume v;
-    int b = 0;
+static void pulse_get_volume (int * l, int * r)
+{
+    * l = * r = 0;
 
-    *l = *r = 100;
+    if (! connected || ! volume_valid)
+        return;
 
-    if (connected) {
-        pa_threaded_mainloop_lock(mainloop);
-        CHECK_DEAD_GOTO(fail, 1);
+    pa_threaded_mainloop_lock (mainloop);
+    CHECK_DEAD_GOTO (fail, 1);
 
-        v = volume;
-        b = volume_valid;
-
-    fail:
-        pa_threaded_mainloop_unlock(mainloop);
-    } else {
-        v = volume;
-        b = volume_valid;
+    if (volume.channels == 2)
+    {
+        * l = (volume.values[0] * 100 + PA_VOLUME_NORM / 2) / PA_VOLUME_NORM;
+        * r = (volume.values[1] * 100 + PA_VOLUME_NORM / 2) / PA_VOLUME_NORM;
     }
+    else
+        * l = * r = (pa_cvolume_avg (& volume) * 100 + PA_VOLUME_NORM / 2) /
+         PA_VOLUME_NORM;
 
-    if (b) {
-        if (v.channels == 2) {
-            *l = (int) ((v.values[0]*100)/PA_VOLUME_NORM);
-            *r = (int) ((v.values[1]*100)/PA_VOLUME_NORM);
-        } else
-            *l = *r = (int) ((pa_cvolume_avg(&v)*100)/PA_VOLUME_NORM);
-    }
+fail:
+    pa_threaded_mainloop_unlock (mainloop);
 }
 
 static void volume_time_cb(pa_mainloop_api *api, pa_time_event *e, const struct timeval *tv, void *userdata) {
     pa_operation *o;
 
     if (!(o = pa_context_set_sink_input_volume(context, pa_stream_get_index(stream), &volume, NULL, NULL)))
-        g_warning("pa_context_set_sink_input_volume() failed: %s", pa_strerror(pa_context_errno(context)));
+        AUDDBG("pa_context_set_sink_input_volume() failed: %s", pa_strerror(pa_context_errno(context)));
     else
         pa_operation_unref(o);
 
@@ -221,17 +202,22 @@ static void volume_time_cb(pa_mainloop_api *api, pa_time_event *e, const struct
 
 static void pulse_set_volume(int l, int r) {
 
-    if (connected) {
-        pa_threaded_mainloop_lock(mainloop);
-        CHECK_DEAD_GOTO(fail, 1);
-    }
+    if (! connected)
+        return;
+
+    pa_threaded_mainloop_lock (mainloop);
+    CHECK_DEAD_GOTO (fail, 1);
+
+    /* sanitize output volumes. */
+    l = CLAMP(l, 0, 100);
+    r = CLAMP(r, 0, 100);
 
     if (!volume_valid || volume.channels !=  1) {
-        volume.values[0] = ((pa_volume_t) l * PA_VOLUME_NORM)/100;
-        volume.values[1] = ((pa_volume_t) r * PA_VOLUME_NORM)/100;
+        volume.values[0] = ((pa_volume_t) l * PA_VOLUME_NORM + 50) / 100;
+        volume.values[1] = ((pa_volume_t) r * PA_VOLUME_NORM + 50) / 100;
         volume.channels = 2;
     } else {
-        volume.values[0] = ((pa_volume_t) l * PA_VOLUME_NORM)/100;
+        volume.values[0] = ((pa_volume_t) MAX (l, r) * PA_VOLUME_NORM + 50) / 100;
         volume.channels = 1;
     }
 
@@ -248,7 +234,8 @@ fail:
         pa_threaded_mainloop_unlock(mainloop);
 }
 
-static void pulse_pause(short b) {
+static void pulse_pause (gboolean b)
+{
     pa_operation *o = NULL;
     int success = 0;
 
@@ -258,7 +245,7 @@ static void pulse_pause(short b) {
     CHECK_DEAD_GOTO(fail, 1);
 
     if (!(o = pa_stream_cork(stream, b, stream_success_cb, &success))) {
-        g_warning("pa_stream_cork() failed: %s", pa_strerror(pa_context_errno(context)));
+        AUDDBG("pa_stream_cork() failed: %s", pa_strerror(pa_context_errno(context)));
         goto fail;
     }
 
@@ -268,7 +255,7 @@ static void pulse_pause(short b) {
     }
 
     if (!success)
-        g_warning("pa_stream_cork() failed: %s", pa_strerror(pa_context_errno(context)));
+        AUDDBG("pa_stream_cork() failed: %s", pa_strerror(pa_context_errno(context)));
 
 fail:
 
@@ -288,7 +275,7 @@ static int pulse_free(void) {
     CHECK_DEAD_GOTO(fail, 1);
 
     if ((l = pa_stream_writable_size(stream)) == (size_t) -1) {
-        g_warning("pa_stream_writable_size() failed: %s", pa_strerror(pa_context_errno(context)));
+        AUDDBG("pa_stream_writable_size() failed: %s", pa_strerror(pa_context_errno(context)));
         l = 0;
         goto fail;
     }
@@ -299,7 +286,7 @@ static int pulse_free(void) {
         int success = 0;
 
         if (!(o = pa_stream_trigger(stream, stream_success_cb, &success))) {
-            g_warning("pa_stream_trigger() failed: %s", pa_strerror(pa_context_errno(context)));
+            AUDDBG("pa_stream_trigger() failed: %s", pa_strerror(pa_context_errno(context)));
             goto fail;
         }
 
@@ -309,7 +296,7 @@ static int pulse_free(void) {
         }
 
         if (!success)
-            g_warning("pa_stream_trigger() failed: %s", pa_strerror(pa_context_errno(context)));
+            AUDDBG("pa_stream_trigger() failed: %s", pa_strerror(pa_context_errno(context)));
     }
 
 fail:
@@ -330,7 +317,7 @@ static int pulse_get_written_time(void) {
     pa_threaded_mainloop_lock(mainloop);
     CHECK_DEAD_GOTO(fail, 1);
 
-    r = (int) (((double) written*1000) / pa_bytes_per_second(pa_stream_get_sample_spec(stream)));
+    r = written * (int64_t) 1000 / bytes_per_second;
 
 fail:
     pa_threaded_mainloop_unlock(mainloop);
@@ -338,71 +325,69 @@ fail:
     return r;
 }
 
-static int pulse_get_output_time(void) {
-    int r = 0;
-    pa_usec_t t;
+static void pulse_set_written_time (int time)
+{
+    pa_threaded_mainloop_lock (mainloop);
 
-    CHECK_CONNECTED(0);
+    written = time * (int64_t) bytes_per_second / 1000;
+    flush_time = time;
 
-    pa_threaded_mainloop_lock(mainloop);
-
-    for (;;) {
-        CHECK_DEAD_GOTO(fail, 1);
-
-        if (pa_stream_get_time(stream, &t) >= 0)
-            break;
-
-        if (pa_context_errno(context) != PA_ERR_NODATA) {
-            g_warning("pa_stream_get_time() failed: %s", pa_strerror(pa_context_errno(context)));
-            goto fail;
-        }
-
-        pa_threaded_mainloop_wait(mainloop);
-    }
-
-    r = (int) (t / 1000);
-
-    if (just_flushed) {
-        time_offset_msec -= r;
-        just_flushed = 0;
-    }
-
-    r += time_offset_msec;
-
-fail:
-    pa_threaded_mainloop_unlock(mainloop);
-
-    return r;
+    pa_threaded_mainloop_unlock (mainloop);
 }
 
-static int pulse_playing(void) {
-    int r = 0;
-    const pa_timing_info *i;
+static int pulse_get_output_time (void)
+{
+    int time = 0;
 
     CHECK_CONNECTED(0);
 
     pa_threaded_mainloop_lock(mainloop);
 
-    for (;;) {
+    time = written * (int64_t) 1000 / bytes_per_second;
+
+    pa_usec_t usec;
+    int neg;
+    if (pa_stream_get_latency (stream, & usec, & neg) == PA_OK)
+        time -= usec / 1000;
+
+    /* fix for AUDPLUG-308: pa_stream_get_latency() still returns positive even
+     * immediately after a flush; fix the result so that we don't return less
+     * than the flush time */
+    if (time < flush_time)
+        time = flush_time;
+
+    pa_threaded_mainloop_unlock(mainloop);
+
+    return time;
+}
+
+static void pulse_drain(void) {
+    pa_operation *o = NULL;
+    int success = 0;
+
+    CHECK_CONNECTED();
+
+    pa_threaded_mainloop_lock(mainloop);
+    CHECK_DEAD_GOTO(fail, 0);
+
+    if (!(o = pa_stream_drain(stream, stream_success_cb, &success))) {
+        AUDDBG("pa_stream_drain() failed: %s", pa_strerror(pa_context_errno(context)));
+        goto fail;
+    }
+
+    while (pa_operation_get_state(o) != PA_OPERATION_DONE) {
         CHECK_DEAD_GOTO(fail, 1);
-
-        if ((i = pa_stream_get_timing_info(stream)))
-            break;
-
-        if (pa_context_errno(context) != PA_ERR_NODATA) {
-            g_warning("pa_stream_get_timing_info() failed: %s", pa_strerror(pa_context_errno(context)));
-            goto fail;
-        }
-
         pa_threaded_mainloop_wait(mainloop);
     }
 
-    r = i->playing;
+    if (!success)
+        AUDDBG("pa_stream_drain() failed: %s", pa_strerror(pa_context_errno(context)));
 
 fail:
-    pa_threaded_mainloop_unlock(mainloop);
+    if (o)
+        pa_operation_unref(o);
 
-    return r;
+    pa_threaded_mainloop_unlock(mainloop);
 }
 
 static void pulse_flush(int time) {
@@ -414,13 +399,11 @@ static void pulse_flush(int time) {
     pa_threaded_mainloop_lock(mainloop);
     CHECK_DEAD_GOTO(fail, 1);
 
-    /* gapless playback: new stream, reset the title. --nenolod */
-    if (time == 0) {
-        pa_stream_set_name(stream, get_song_name(), stream_success_cb, &success);
-    }
+    written = time * (int64_t) bytes_per_second / 1000;
+    flush_time = time;
 
     if (!(o = pa_stream_flush(stream, stream_success_cb, &success))) {
-        g_warning("pa_stream_flush() failed: %s", pa_strerror(pa_context_errno(context)));
+        AUDDBG("pa_stream_flush() failed: %s", pa_strerror(pa_context_errno(context)));
         goto fail;
     }
 
@@ -430,11 +413,7 @@ static void pulse_flush(int time) {
     }
 
     if (!success)
-        g_warning("pa_stream_flush() failed: %s", pa_strerror(pa_context_errno(context)));
-
-    written = (uint64_t) (((double) time * pa_bytes_per_second(pa_stream_get_sample_spec(stream))) / 1000);
-    just_flushed = 1;
-    time_offset_msec = time;
+        AUDDBG("pa_stream_flush() failed: %s", pa_strerror(pa_context_errno(context)));
 
 fail:
     if (o)
@@ -445,7 +424,6 @@ fail:
 
 static void pulse_write(void* ptr, int length) {
     gint writeoffs, remain, writable;
-    gint fragsize = 1024; /* TODO: make fragment size configurable */
 
     CHECK_CONNECTED();
 
@@ -460,13 +438,14 @@ static void pulse_write(void* ptr, int length) {
          gpointer pptr = ptr + writeoffs;
 
          writable = length - writeoffs;
+         size_t fragsize = pa_stream_writable_size(stream);
 
-         /* don't write any more than a fragment the size of fragsize at a time. */
+         /* don't write more than what PA is willing to handle right now. */
          if (writable > fragsize)
              writable = fragsize;
 
          if (pa_stream_write(stream, pptr, writable, NULL, PA_SEEK_RELATIVE, 0) < 0) {
-             g_warning("pa_stream_write() failed: %s", pa_strerror(pa_context_errno(context)));
+             AUDDBG("pa_stream_write() failed: %s", pa_strerror(pa_context_errno(context)));
              goto fail;
          }
     }
@@ -478,39 +457,8 @@ fail:
     pa_threaded_mainloop_unlock(mainloop);
 }
 
-static void drain(void) {
-    pa_operation *o = NULL;
-    int success = 0;
-
-    CHECK_CONNECTED();
-
-    pa_threaded_mainloop_lock(mainloop);
-    CHECK_DEAD_GOTO(fail, 0);
-
-    if (!(o = pa_stream_drain(stream, stream_success_cb, &success))) {
-        g_warning("pa_stream_drain() failed: %s", pa_strerror(pa_context_errno(context)));
-        goto fail;
-    }
-
-    while (pa_operation_get_state(o) != PA_OPERATION_DONE) {
-        CHECK_DEAD_GOTO(fail, 1);
-        pa_threaded_mainloop_wait(mainloop);
-    }
-
-    if (!success)
-        g_warning("pa_stream_drain() failed: %s", pa_strerror(pa_context_errno(context)));
-
-fail:
-    if (o)
-        pa_operation_unref(o);
-
-    pa_threaded_mainloop_unlock(mainloop);
-}
-
 static void pulse_close(void)
 {
-    drain();
-
     connected = 0;
 
     if (mainloop)
@@ -534,9 +482,10 @@ static void pulse_close(void)
     }
 
     volume_time_event = NULL;
+    volume_valid = 0;
 }
 
-static int pulse_open(AFormat fmt, int rate, int nch) {
+static int pulse_open(gint fmt, int rate, int nch) {
     pa_sample_spec ss;
     pa_operation *o = NULL;
     int success;
@@ -557,28 +506,27 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
         case FMT_S16_BE:
             ss.format = PA_SAMPLE_S16BE;
             break;
-        case FMT_S16_NE:
-            ss.format = PA_SAMPLE_S16NE;
-            break;
-#if 0 /* pulseaudio only handles 16 and 32bit formats*/
+
+#ifdef PA_SAMPLE_S24_32LE
         case FMT_S24_LE:
-            ss.format = PA_SAMPLE_S24LE;
+            ss.format = PA_SAMPLE_S24_32LE;
             break;
         case FMT_S24_BE:
-            ss.format = PA_SAMPLE_S24BE;
-            break;
-        case FMT_S24_NE:
-            ss.format = PA_SAMPLE_S24NE;
+            ss.format = PA_SAMPLE_S24_32BE;
             break;
 #endif
+
+#ifdef PA_SAMPLE_S32LE
         case FMT_S32_LE:
             ss.format = PA_SAMPLE_S32LE;
             break;
         case FMT_S32_BE:
             ss.format = PA_SAMPLE_S32BE;
             break;
-        case FMT_S32_NE:
-            ss.format = PA_SAMPLE_S32NE;
+#endif
+
+	case FMT_FLOAT:
+            ss.format = PA_SAMPLE_FLOAT32NE;
             break;
         default:
             return FALSE;
@@ -589,21 +537,15 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     if (!pa_sample_spec_valid(&ss))
         return FALSE;
 
-    if (!volume_valid) {
-        pa_cvolume_reset(&volume, ss.channels);
-        volume_valid = 1;
-    } else if (volume.channels != ss.channels)
-        pa_cvolume_set(&volume, ss.channels, pa_cvolume_avg(&volume));
-
     if (!(mainloop = pa_threaded_mainloop_new())) {
-        g_warning("Failed to allocate main loop");
+        ERROR ("Failed to allocate main loop");
         goto fail;
     }
 
     pa_threaded_mainloop_lock(mainloop);
 
     if (!(context = pa_context_new(pa_threaded_mainloop_get_api(mainloop), "Audacious"))) {
-        g_warning("Failed to allocate context");
+        ERROR ("Failed to allocate context");
         goto unlock_and_fail;
     }
 
@@ -611,12 +553,12 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     pa_context_set_subscribe_callback(context, subscribe_cb, NULL);
 
     if (pa_context_connect(context, NULL, 0, NULL) < 0) {
-        g_warning("Failed to connect to server: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("Failed to connect to server: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
     if (pa_threaded_mainloop_start(mainloop) < 0) {
-        g_warning("Failed to start main loop");
+        ERROR ("Failed to start main loop");
         goto unlock_and_fail;
     }
 
@@ -624,12 +566,12 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     pa_threaded_mainloop_wait(mainloop);
 
     if (pa_context_get_state(context) != PA_CONTEXT_READY) {
-        g_warning("Failed to connect to server: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("Failed to connect to server: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
-    if (!(stream = pa_stream_new(context, get_song_name(), &ss, NULL))) {
-        g_warning("Failed to create stream: %s", pa_strerror(pa_context_errno(context)));
+    if (!(stream = pa_stream_new(context, "Audacious", &ss, NULL))) {
+        ERROR ("Failed to create stream: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -637,8 +579,9 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     pa_stream_set_write_callback(stream, stream_request_cb, NULL);
     pa_stream_set_latency_update_callback(stream, stream_latency_update_cb, NULL);
 
-    if (pa_stream_connect_playback(stream, NULL, NULL, PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE, &volume, NULL) < 0) {
-        g_warning("Failed to connect stream: %s", pa_strerror(pa_context_errno(context)));
+    /* Connect stream with sink and default volume */
+    if (pa_stream_connect_playback(stream, NULL, NULL, PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE, NULL, NULL) < 0) {
+        ERROR ("Failed to connect stream: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -646,13 +589,13 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     pa_threaded_mainloop_wait(mainloop);
 
     if (pa_stream_get_state(stream) != PA_STREAM_READY) {
-        g_warning("Failed to connect stream: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("Failed to connect stream: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
     /* Now subscribe to events */
     if (!(o = pa_context_subscribe(context, PA_SUBSCRIPTION_MASK_SINK_INPUT, context_success_cb, &success))) {
-        g_warning("pa_context_subscribe() failed: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("pa_context_subscribe() failed: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -663,7 +606,7 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     }
 
     if (!success) {
-        g_warning("pa_context_subscribe() failed: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("pa_context_subscribe() failed: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -671,7 +614,7 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
 
     /* Now request the initial stream info */
     if (!(o = pa_context_get_sink_input_info(context, pa_stream_get_index(stream), info_cb, NULL))) {
-        g_warning("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -681,14 +624,15 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     }
 
     if (!volume_valid) {
-        g_warning("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
+    pa_operation_unref(o);
 
     do_trigger = 0;
     written = 0;
-    time_offset_msec = 0;
-    just_flushed = 0;
+    flush_time = 0;
+    bytes_per_second = FMT_SIZEOF (fmt) * nch * rate;
     connected = 1;
     volume_time_event = NULL;
 
@@ -710,15 +654,20 @@ fail:
     return FALSE;
 }
 
+static gboolean pulse_init (void)
+{
+    if (! pulse_open (FMT_S16_NE, 44100, 2))
+        return FALSE;
+
+    pulse_close ();
+    return TRUE;
+}
+
 static void pulse_about(void) {
     static GtkWidget *dialog;
-
-    if (dialog != NULL)
-        return;
-
-    dialog = audacious_info_dialog(
+    audgui_simple_message(& dialog, GTK_MESSAGE_INFO,
             _("About Audacious PulseAudio Output Plugin"),
-            _("Audacious PulseAudio Output Plugin\n\n "
+            "Audacious PulseAudio Output Plugin\n\n "
             "This program is free software; you can redistribute it and/or modify\n"
             "it under the terms of the GNU General Public License as published by\n"
             "the Free Software Foundation; either version 2 of the License, or\n"
@@ -732,21 +681,14 @@ static void pulse_about(void) {
             "You should have received a copy of the GNU General Public License\n"
             "along with this program; if not, write to the Free Software\n"
             "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,\n"
-            "USA."),
-            _("OK"),
-            FALSE,
-            NULL,
-            NULL);
-
-    gtk_signal_connect(
-            GTK_OBJECT(dialog),
-            "destroy",
-            GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-            &dialog);
+            "USA.");
 }
 
-static OutputPlugin pulse_op = {
-        .description = "PulseAudio Output Plugin",
+AUD_OUTPUT_PLUGIN
+(
+        .name = "PulseAudio",
+        .probe_priority = 8,
+        .init = pulse_init,
         .about = pulse_about,
         .get_volume = pulse_get_volume,
         .set_volume = pulse_set_volume,
@@ -756,11 +698,8 @@ static OutputPlugin pulse_op = {
         .flush = pulse_flush,
         .pause = pulse_pause,
         .buffer_free = pulse_free,
-        .buffer_playing = pulse_playing,
+        .drain = pulse_drain,
         .output_time = pulse_get_output_time,
         .written_time = pulse_get_written_time,
-};
-
-OutputPlugin *pulse_oplist[] = { &pulse_op, NULL };
-
-SIMPLE_OUTPUT_PLUGIN(pulser, pulse_oplist);
+        .set_written_time = pulse_set_written_time,
+)
