@@ -1,20 +1,20 @@
+#include <inttypes.h>
 #include <pthread.h>
-#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-#include <curl/curl.h>
 #include <stdio.h>
 #include "fmt.h"
-#include "md5.h"
+#include "plugin.h"
 #include "scrobbler.h"
-#include "config.h"
 #include "settings.h"
 #include <glib.h>
 
+#include <audacious/drct.h>
+#include <audacious/debug.h>
+#include <audacious/misc.h>
 #include <audacious/plugin.h>
 
-#define SCROBBLER_HS_URL "http://post.audioscrobbler.com"
 #define SCROBBLER_CLI_ID "aud"
 #define SCROBBLER_HS_WAIT 1800
 #define SCROBBLER_SB_WAIT 10
@@ -38,6 +38,7 @@ static int  sc_hs_status,
 
 static char     *sc_submit_url, /* queue */
         *sc_np_url, /* np */
+        *sc_hs_url, /* handshake url */
         *sc_session_id,
         *sc_username = NULL,
         *sc_password = NULL,
@@ -120,7 +121,7 @@ static void q_item_free(item_t *item)
 
 static item_t *q_additem(item_t *newitem)
 {
-    pdebug(fmt_vastr("Adding %s - %s to the queue", newitem->artist, newitem->title), DEBUG);
+    AUDDBG("Adding %s - %s to the queue\n", newitem->artist, newitem->title);
 
     q_nitems++;
     newitem->next = NULL;
@@ -139,20 +140,35 @@ static item_t *q_additem(item_t *newitem)
 static item_t *create_item(Tuple *tuple, int len)
 {
     item_t *item;
-    const gchar *album;
+    gchar *album, *artist, *title;
 
     item = malloc(sizeof(item_t));
 
-    item->artist = fmt_escape(aud_tuple_get_string(tuple, FIELD_ARTIST, NULL));
-    item->title = fmt_escape(aud_tuple_get_string(tuple, FIELD_TITLE, NULL));
+    artist = tuple_get_str(tuple, FIELD_ARTIST, NULL);
+    item->artist = fmt_escape(artist);
+    str_unref(artist);
+
+    title = tuple_get_str(tuple, FIELD_TITLE, NULL);
+    item->title = fmt_escape(title);
+    str_unref(title);
+
+    if (item->artist == NULL || item->title == NULL)
+    {
+        free (item);
+        return NULL;
+    }
+
     item->len = len;
-    item->track = aud_tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL);
+    item->track = tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL);
     item->timeplayed = 0;
     item->utctime = time(NULL);
 
-    album = aud_tuple_get_string(tuple, FIELD_ALBUM, NULL);
+    album = tuple_get_str(tuple, FIELD_ALBUM, NULL);
     if (album)
-        item->album = fmt_escape((char*) album);
+    {
+        item->album = fmt_escape(album);
+        str_unref(album);
+    }
     else
         item->album = fmt_escape("");
 
@@ -165,7 +181,9 @@ static item_t *q_put(Tuple *tuple, int t, int len)
 {
     item_t *item;
 
-    item = create_item(tuple, len);
+    if ((item = create_item (tuple, len)) == NULL)
+        return NULL;
+
     item->timeplayed = len;
     item->utctime = t;
 
@@ -174,10 +192,12 @@ static item_t *q_put(Tuple *tuple, int t, int len)
 
 static item_t *set_np(Tuple *tuple, int len)
 {
-    q_item_free(np_item);
-    np_item = create_item(tuple, len);
+    if (np_item != NULL)
+        q_item_free (np_item);
 
-    pdebug(fmt_vastr("Tracking now-playing track: %s - %s", np_item->artist, np_item->title), DEBUG);
+    if ((np_item = create_item (tuple, len)) != NULL)
+        AUDDBG ("Tracking now-playing track: %s - %s\n", np_item->artist,
+         np_item->title);
 
     return np_item;
 }
@@ -215,7 +235,7 @@ static int q_get(void)
 
     if (q_nitems == 0)
         return 0;
-    
+
     item = q_queue;
 
     if(item == NULL)
@@ -224,7 +244,7 @@ static int q_get(void)
     q_nitems--;
     q_queue = q_queue->next;
 
-    pdebug(fmt_vastr("Removing %s - %s from queue", item->artist, item->title), DEBUG);
+    AUDDBG("Removing %s - %s from queue\n", item->artist, item->title);
 
     q_item_free(item);
 
@@ -242,16 +262,13 @@ static void q_free(void)
     while (q_get());
 }
 
-static int q_len(void)
-{
-    return q_nitems;
-}
 
 /* isn't there better way for that? --desowin */
-gboolean sc_timeout(gpointer data) {
+gboolean sc_timeout(gpointer data)
+{
     if (np_item)
     {
-        if (audacious_drct_get_playing() && !audacious_drct_get_paused())
+        if (aud_drct_get_playing() && !aud_drct_get_paused())
             np_item->timeplayed+=1;
 
         /*
@@ -260,6 +277,8 @@ gboolean sc_timeout(gpointer data) {
         if (((np_item->timeplayed >= (np_item->len / 2)) ||
             (np_item->timeplayed >= 240)))
         {
+            AUDDBG("submitting!!!\n");
+
             q_additem(np_item);
             np_item = NULL;
             dump_queue();
@@ -326,11 +345,16 @@ static int sc_parse_hs_res(void)
     char *interval;
 
     if (!sc_srv_res_size) {
-        pdebug("No reply from server", DEBUG);
+        AUDDBG("No reply from server\n");
+        return -1;
+    }
+    if (sc_srv_res == NULL) {
+        AUDDBG("Reply is NULL, size=%d\n", sc_srv_res_size);
         return -1;
     }
     *(sc_srv_res + sc_srv_res_size) = 0;
 
+    AUDDBG("reply is: %s\n", sc_srv_res);
     if (!strncmp(sc_srv_res, "OK\n", 3)) {
         gchar *scratch = g_strdup(sc_srv_res);
         gchar **split = g_strsplit(scratch, "\n", 5);
@@ -350,7 +374,7 @@ static int sc_parse_hs_res(void)
         /* Throwing a major error, just in case */
         /* sc_throw_error(fmt_vastr("%s", sc_srv_res));
            sc_hs_errors++; */
-        pdebug(fmt_vastr("error: %s", sc_srv_res), DEBUG);
+        AUDDBG("error: %s\n", sc_srv_res);
 
         return -1;
     }
@@ -359,7 +383,7 @@ static int sc_parse_hs_res(void)
         interval = strstr(sc_srv_res, "INTERVAL");
         if(!interval)
         {
-            pdebug("missing INTERVAL", DEBUG);
+            AUDDBG("missing INTERVAL\n");
         }
         else
         {
@@ -377,7 +401,7 @@ static int sc_parse_hs_res(void)
         /* Throwing major error. Need to alert client to update. */
         sc_throw_error(fmt_vastr("Please update Audacious.\n"
             "Update available at: http://audacious-media-player.org"));
-        pdebug(fmt_vastr("update client: %s", sc_srv_res + 7), DEBUG);
+        AUDDBG("update client: %s\n", sc_srv_res + 7);
 
         /*
          * Russ isn't clear on whether we can submit with a not-updated
@@ -391,7 +415,7 @@ static int sc_parse_hs_res(void)
 
         interval = strstr(sc_srv_res, "INTERVAL");
         if (!interval) {
-            pdebug("missing INTERVAL", DEBUG);
+            AUDDBG("missing INTERVAL\n");
             /*
              * This is probably a bad thing, but Russ seems to
              * think its OK to assume that an UPTODATE response
@@ -418,12 +442,12 @@ static int sc_parse_hs_res(void)
         /* Throwing major error. */
         sc_throw_error("Incorrect username/password.\n"
                 "Please fix in configuration.");
-        pdebug("incorrect username/password", DEBUG);
+        AUDDBG("incorrect username/password\n");
 
         interval = strstr(sc_srv_res, "INTERVAL");
         if(!interval)
         {
-            pdebug("missing INTERVAL", DEBUG);
+            AUDDBG("missing INTERVAL\n");
         }
         else
         {
@@ -434,18 +458,19 @@ static int sc_parse_hs_res(void)
         return -1;
     }
 
-    pdebug(fmt_vastr("unknown server-reply '%s'", sc_srv_res), DEBUG);
+    AUDDBG("unknown server-reply '%s'\n", sc_srv_res);
     return -1;
 }
 
 static unsigned char *md5_string(char *pass, int len)
 {
-    md5_state_t md5state;
     static unsigned char md5pword[16];
-        
-    md5_init(&md5state);
-    md5_append(&md5state, (unsigned const char *)pass, len);
-    md5_finish(&md5state, md5pword);
+    gsize md5len = 16;
+
+    GChecksum * state = g_checksum_new (G_CHECKSUM_MD5);
+    g_checksum_update (state, (unsigned char *) pass, len);
+    g_checksum_get_digest (state, md5pword, & md5len);
+    g_checksum_free (state);
 
     return md5pword;
 }
@@ -457,7 +482,7 @@ static void hexify(char *pass, int len)
     int i;
 
     memset(sc_response_hash, 0, sizeof(sc_response_hash));
-    
+
     for(i = 0; i < len; i++) {
         *(bp++) = hexchars[(pass[i] >> 4) & 0x0f];
         *(bp++) = hexchars[pass[i] & 0x0f];
@@ -467,6 +492,44 @@ static void hexify(char *pass, int len)
     return;
 }
 
+static int sc_parse_sb_res(void);
+static GStaticMutex submit_mutex = G_STATIC_MUTEX_INIT;
+
+gpointer sc_curl_perform_thread(gpointer data)
+{
+    CURL *curl = (CURL *) data;
+
+    g_static_mutex_lock(&submit_mutex);
+
+    curl_easy_perform(curl);
+    curl_easy_cleanup(curl);
+
+    if (sc_parse_sb_res()) {
+        sc_sb_errors++;
+        sc_free_res();
+        AUDDBG("Retrying in %d secs, %d elements in queue\n",
+                    sc_submit_interval, q_nitems);
+
+        g_static_mutex_unlock(&submit_mutex);
+
+        g_thread_exit(NULL);
+        return NULL;
+    }
+
+    g_static_mutex_unlock(&submit_mutex);
+
+    sc_free_res();
+    g_thread_exit(NULL);
+    return NULL;
+}
+
+void sc_curl_perform(CURL *curl)
+{
+    GError **error = NULL;
+
+    g_thread_create(sc_curl_perform_thread, curl, FALSE, error);
+}
+
 static int sc_handshake(void)
 {
     int status;
@@ -476,26 +539,27 @@ static int sc_handshake(void)
     char *auth_tmp;
     char *auth;
 
-    auth_tmp = g_strdup_printf("%s%ld", sc_password, ts);
+    auth_tmp = g_strdup_printf("%s%"PRId64, sc_password, (gint64) ts);
     auth = (char *)md5_string(auth_tmp, strlen(auth_tmp));
     g_free(auth_tmp);
-    hexify(auth, strlen(auth));
+    hexify(auth, 16);
     auth_tmp = g_strdup(sc_response_hash);
 
-    g_snprintf(buf, sizeof(buf), "%s/?hs=true&p=%s&c=%s&v=%s&u=%s&t=%ld&a=%s",
-            SCROBBLER_HS_URL, SCROBBLER_VERSION,
-            SCROBBLER_CLI_ID, SCROBBLER_IMPLEMENTATION, sc_username, time(NULL),
+    g_snprintf(buf, sizeof(buf), "%s/?hs=true&p=%s&c=%s&v=%s&u=%s&t=%"PRId64"&a=%s",
+            sc_hs_url, SCROBBLER_VERSION,
+            SCROBBLER_CLI_ID, SCROBBLER_IMPLEMENTATION, sc_username, (gint64) ts,
             auth_tmp);
     g_free(auth_tmp);
 
     curl = curl_easy_init();
+    setup_proxy(curl);
+    curl_easy_setopt(curl, CURLOPT_HTTPGET, 1);
     curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
     curl_easy_setopt(curl, CURLOPT_URL, buf);
-    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, 
+    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION,
             sc_store_res);
     memset(sc_curl_errbuf, 0, sizeof(sc_curl_errbuf));
     curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, sc_curl_errbuf);
-    curl_easy_setopt(curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
     curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
     curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, SC_CURL_TIMEOUT);
     status = curl_easy_perform(curl);
@@ -504,7 +568,7 @@ static int sc_handshake(void)
     sc_hs_timeout = time(NULL) + SCROBBLER_HS_WAIT;
 
     if (status) {
-        pdebug(sc_curl_errbuf, DEBUG);
+        AUDDBG("curl error: %s\n", sc_curl_errbuf);
         sc_hs_errors++;
         sc_free_res();
         return -1;
@@ -517,19 +581,21 @@ static int sc_handshake(void)
     }
 
     if (sc_challenge_hash != NULL) {
-        md5_state_t md5state;
         unsigned char md5pword[16];
-        
-        md5_init(&md5state);
-        /*pdebug(fmt_vastr("Pass Hash: %s", sc_password), DEBUG);*/
-        md5_append(&md5state, (unsigned const char *)sc_password,
-                strlen(sc_password));
-        /*pdebug(fmt_vastr("Challenge Hash: %s", sc_challenge_hash), DEBUG);*/
-        md5_append(&md5state, (unsigned const char *)sc_challenge_hash,
-                strlen(sc_challenge_hash));
-        md5_finish(&md5state, md5pword);
+        gsize md5len = 16;
+
+        /*AUDDBG("Pass Hash: %s", sc_password));*/
+        /*AUDDBG("Challenge Hash: %s", sc_challenge_hash));*/
+
+        GChecksum * state = g_checksum_new (G_CHECKSUM_MD5);
+        g_checksum_update (state, (unsigned char *) sc_password, strlen (sc_password));
+        g_checksum_update (state, (unsigned char *) sc_challenge_hash,
+         strlen (sc_challenge_hash));
+        g_checksum_get_digest (state, md5pword, & md5len);
+        g_checksum_free (state);
+
         hexify((char*)md5pword, sizeof(md5pword));
-        /*pdebug(fmt_vastr("Response Hash: %s", sc_response_hash), DEBUG);*/
+        /*AUDDBG("Response Hash: %s", sc_response_hash));*/
     }
 
     sc_hs_errors = 0;
@@ -537,8 +603,8 @@ static int sc_handshake(void)
 
     sc_free_res();
 
-    pdebug(fmt_vastr("submiturl: %s - interval: %d", 
-                sc_submit_url, sc_submit_interval), DEBUG);
+    AUDDBG("submiturl: %s - interval: %d\n",
+                sc_submit_url, sc_submit_interval);
 
     return 0;
 }
@@ -548,19 +614,24 @@ static int sc_parse_sb_res(void)
     char *ch, *ch2;
 
     if (!sc_srv_res_size) {
-        pdebug("No response from server", DEBUG);
+        AUDDBG("No response from server\n");
+        return -1;
+    }
+    if (sc_srv_res == NULL) {
+        AUDDBG("Reply is NULL, size=%d\n", sc_srv_res_size);
         return -1;
     }
     *(sc_srv_res + sc_srv_res_size) = 0;
 
+    AUDDBG("message: %s\n", sc_srv_res);
     if (!strncmp(sc_srv_res, "OK", 2)) {
         if ((ch = strstr(sc_srv_res, "INTERVAL"))) {
             sc_submit_interval = strtol(ch + 8, NULL, 10);
-            pdebug(fmt_vastr("got new interval: %d",
-                        sc_submit_interval), DEBUG);
+            AUDDBG("got new interval: %d\n",
+                        sc_submit_interval);
         }
 
-        pdebug(fmt_vastr("submission ok: %s", sc_srv_res), DEBUG);
+        AUDDBG("submission ok: %s\n", sc_srv_res);
 
         return 0;
     }
@@ -568,11 +639,11 @@ static int sc_parse_sb_res(void)
     if (!strncmp(sc_srv_res, "BADAUTH", 7)) {
         if ((ch = strstr(sc_srv_res, "INTERVAL"))) {
             sc_submit_interval = strtol(ch + 8, NULL, 10);
-            pdebug(fmt_vastr("got new interval: %d",
-                        sc_submit_interval), DEBUG);
+            AUDDBG("got new interval: %d\n",
+                        sc_submit_interval);
         }
 
-        pdebug("incorrect username/password", DEBUG);
+        AUDDBG("incorrect username/password\n");
 
         sc_giveup = 0;
 
@@ -594,8 +665,8 @@ static int sc_parse_sb_res(void)
 
         if(sc_bad_users > 2)
         {
-            pdebug("3 BADAUTH returns on submission. Halting "
-                "submissions until login fixed.", DEBUG)
+            AUDDBG("3 BADAUTH returns on submission. Halting "
+                "submissions until login fixed.\n");
             sc_throw_error("Incorrect username/password.\n"
                 "Please fix in configuration.");
         }
@@ -603,17 +674,26 @@ static int sc_parse_sb_res(void)
         return -1;
     }
 
+    if(!strncmp(sc_srv_res, "BADSESSION", 10)) {
+        AUDDBG("Invalid session, re-handshaking\n");
+
+        sc_free_res();
+        sc_handshake();
+
+        return -1;
+    }
+
     if (!strncmp(sc_srv_res, "FAILED", 6))  {
         if ((ch = strstr(sc_srv_res, "INTERVAL"))) {
             sc_submit_interval = strtol(ch + 8, NULL, 10);
-            pdebug(fmt_vastr("got new interval: %d",
-                        sc_submit_interval), DEBUG);
+            AUDDBG("got new interval: %d\n",
+                        sc_submit_interval);
         }
 
         /* This could be important. (Such as FAILED - Get new plugin) */
         /*sc_throw_error(fmt_vastr("%s", sc_srv_res));*/
 
-        pdebug(sc_srv_res, DEBUG);
+        AUDDBG("%s\n", sc_srv_res);
 
         return -1;
     }
@@ -625,22 +705,22 @@ static int sc_parse_sb_res(void)
             ch += strlen("<TITLE>");
             *ch2 = '\0';
 
-            pdebug(fmt_vastr("HTTP Error (%d): '%s'",
-                     atoi(ch), ch + 4), DEBUG);
+            AUDDBG("HTTP Error (%d): '%s'\n",
+                     atoi(ch), ch + 4);
 //          *ch2 = '<'; // needed? --yaz
         }
 
         return -1;
     }
 
-    pdebug(fmt_vastr("unknown server-reply %s", sc_srv_res), DEBUG);
+    AUDDBG("unknown server-reply %s\n", sc_srv_res);
 
     return -1;
 }
 
 static gchar *sc_itemtag(char c, int n, char *str)
 {
-    static char buf[SCROBBLER_SB_MAXLEN]; 
+    static char buf[SCROBBLER_SB_MAXLEN];
     g_snprintf(buf, SCROBBLER_SB_MAXLEN, "&%c[%d]=%s", c, n, str);
     return buf;
 }
@@ -665,8 +745,10 @@ static int sc_generateentry(GString *submission)
          * The check occurs in the sc_timeout() function now.
          */
 
-        if (!item)
+        if (!item) {
+            AUDDBG("item = NULL :(\n");
             return i;
+        }
 
                 g_string_append(submission,sc_itemtag('a',i,I_ARTIST(item)));
                 g_string_append(submission,sc_itemtag('t',i,I_TITLE(item)));
@@ -684,12 +766,12 @@ static int sc_generateentry(GString *submission)
                 g_free(tmp);
                 g_string_append(submission,sc_itemtag('r',i,""));
 
-        pdebug(fmt_vastr("a[%d]=%s t[%d]=%s l[%d]=%d i[%d]=%d b[%d]=%s",
+        AUDDBG("a[%d]=%s t[%d]=%s l[%d]=%d i[%d]=%d b[%d]=%s\n",
                 i, I_ARTIST(item),
                 i, I_TITLE(item),
                 i, I_LEN(item),
                 i, I_TIME(item),
-                i, I_ALBUM(item)), DEBUG);
+                i, I_ALBUM(item));
         i++;
     }
 
@@ -700,10 +782,10 @@ static int sc_submit_np(Tuple *tuple)
 {
     CURL *curl;
     /* struct HttpPost *post = NULL , *last = NULL; */
-    int status;
-    gchar *entry;
+    static gchar entry[16384];
 
     curl = curl_easy_init();
+    setup_proxy(curl);
     curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
     curl_easy_setopt(curl, CURLOPT_URL, sc_np_url);
     curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION,
@@ -712,15 +794,24 @@ static int sc_submit_np(Tuple *tuple)
     curl_easy_setopt(curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
     /*cfa(&post, &last, "debug", "failed");*/
 
-        char *field_artist = fmt_escape(aud_tuple_get_string(tuple, FIELD_ARTIST, NULL));
-        char *field_title = fmt_escape(aud_tuple_get_string(tuple, FIELD_TITLE, NULL));
-        char *field_album = aud_tuple_get_string(tuple, FIELD_ALBUM, NULL) ? fmt_escape(aud_tuple_get_string(tuple, FIELD_ALBUM, NULL)) : fmt_escape("");
-    entry = g_strdup_printf("s=%s&a=%s&t=%s&b=%s&l=%d&n=%d&m=", sc_session_id,
+    char *artist = tuple_get_str(tuple, FIELD_ARTIST, NULL);
+    char *field_artist = fmt_escape(artist);
+    str_unref(artist);
+
+    char *title = tuple_get_str(tuple, FIELD_TITLE, NULL);
+    char *field_title = fmt_escape(title);
+    str_unref(title);
+
+    char *album = tuple_get_str(tuple, FIELD_ALBUM, NULL);
+    char *field_album = album ? fmt_escape(album) : fmt_escape("");
+    str_unref(album);
+
+    snprintf(entry, 16384, "s=%s&a=%s&t=%s&b=%s&l=%d&n=%d&m=", sc_session_id,
         field_artist,
         field_title,
         field_album,
-        aud_tuple_get_int(tuple, FIELD_LENGTH, NULL) / 1000,
-        aud_tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL));
+        tuple_get_int(tuple, FIELD_LENGTH, NULL) / 1000,
+        tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL));
         curl_free(field_artist);
         curl_free(field_title);
         curl_free(field_album);
@@ -731,27 +822,8 @@ static int sc_submit_np(Tuple *tuple)
     curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
     curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, SC_CURL_TIMEOUT);
     curl_easy_setopt(curl, CURLOPT_TIMEOUT, SCROBBLER_SB_WAIT);
+    sc_curl_perform(curl);
 
-    status = curl_easy_perform(curl);
-    curl_easy_cleanup(curl);
-
-    g_free(entry);
-
-    if (status) {
-        pdebug(sc_curl_errbuf, DEBUG);
-        sc_sb_errors++;
-        sc_free_res();
-        return -1;
-    }
-
-    if (sc_parse_sb_res()) {
-        sc_sb_errors++;
-        sc_free_res();
-        pdebug(fmt_vastr("Retrying in %d secs, %d elements in queue",
-                    sc_submit_interval, q_len()), DEBUG);
-        return -1;
-    }
-    sc_free_res();
     return 0;
 }
 
@@ -759,10 +831,10 @@ static int sc_submitentry(gchar *entry)
 {
     CURL *curl;
     /* struct HttpPost *post = NULL , *last = NULL; */
-    int status;
-        GString *submission;
+    static gchar sub[16384];
 
     curl = curl_easy_init();
+    setup_proxy(curl);
     curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
     curl_easy_setopt(curl, CURLOPT_URL, sc_submit_url);
     curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION,
@@ -771,38 +843,17 @@ static int sc_submitentry(gchar *entry)
     curl_easy_setopt(curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
     /*cfa(&post, &last, "debug", "failed");*/
 
-    /*pdebug(fmt_vastr("Response Hash: %s", sc_response_hash), DEBUG);*/
-        submission = g_string_new("s=");
-        g_string_append(submission, (gchar *)sc_session_id);
-    g_string_append(submission, entry);
+    /*AUDDBG("Response Hash: %s", sc_response_hash));*/
+    snprintf(sub, 16384, "s=%s%s", sc_session_id, entry);
 
-    curl_easy_setopt(curl, CURLOPT_POSTFIELDS, (char *)submission->str);
+    curl_easy_setopt(curl, CURLOPT_POSTFIELDS, (char *) sub);
     memset(sc_curl_errbuf, 0, sizeof(sc_curl_errbuf));
     curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, sc_curl_errbuf);
     curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
     curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, SC_CURL_TIMEOUT);
     curl_easy_setopt(curl, CURLOPT_TIMEOUT, SCROBBLER_SB_WAIT);
+    sc_curl_perform(curl);
 
-    status = curl_easy_perform(curl);
-
-    curl_easy_cleanup(curl);
-        g_string_free(submission, TRUE);
-
-    if (status) {
-        pdebug(sc_curl_errbuf, DEBUG);
-        sc_sb_errors++;
-        sc_free_res();
-        return -1;
-    }
-
-    if (sc_parse_sb_res()) {
-        sc_sb_errors++;
-        sc_free_res();
-        pdebug(fmt_vastr("Retrying in %d secs, %d elements in queue",
-                    sc_submit_interval, q_len()), DEBUG);
-        return -1;
-    }
-    sc_free_res();
     return 0;
 }
 
@@ -813,10 +864,14 @@ static void sc_handlequeue(GMutex *mutex)
     int wait;
     int i;
 
+    AUDDBG("handle queue\n");
+
     if(sc_submit_timeout < time(NULL) && sc_bad_users < 3)
     {
         submitentry = g_string_new("");
 
+        AUDDBG("ok to handle queue!\n");
+
         g_mutex_lock(mutex);
 
         nsubmit = sc_generateentry(submitentry);
@@ -825,14 +880,14 @@ static void sc_handlequeue(GMutex *mutex)
 
         if (nsubmit > 0)
         {
-            pdebug(fmt_vastr("Number submitting: %d", nsubmit), DEBUG);
-            pdebug(fmt_vastr("Submission: %s", submitentry->str), DEBUG);
+            AUDDBG("Number submitting: %d\n", nsubmit);
+            AUDDBG("Submission: %s\n", submitentry->str);
 
             if(!sc_submitentry(submitentry->str))
             {
                 g_mutex_lock(mutex);
 
-                pdebug(fmt_vastr("Clearing out %d item(s) from the queue", nsubmit), DEBUG);
+                AUDDBG("Clearing out %d item(s) from the queue\n", nsubmit);
                 for (i=0; i<nsubmit; i++)
                 {
                     q_get();
@@ -864,12 +919,11 @@ static void sc_handlequeue(GMutex *mutex)
                         ( ((sc_sb_errors - 5) < 7) ?
                         (60 << (sc_sb_errors-5)) :
                         7200 );
-                
+
                 sc_submit_timeout = time(NULL) + wait;
 
-                pdebug(fmt_vastr("Error while submitting. "
-                    "Retrying after %d seconds.", wait),
-                    DEBUG);
+                AUDDBG("Error while submitting. "
+                    "Retrying after %d seconds.\n", wait);
             }
         }
 
@@ -879,26 +933,22 @@ static void sc_handlequeue(GMutex *mutex)
 
 static void read_cache(void)
 {
-    FILE *fd;
-    char buf[PATH_MAX];
     int i=0;
     item_t *item;
-    gchar* config_datadir;
 
-    config_datadir = audacious_get_localdir();
-    g_snprintf(buf, sizeof(buf), "%s/scrobblerqueue.txt", config_datadir);
-    g_free(config_datadir);
+    gchar * path = g_strconcat (aud_get_path (AUD_PATH_USER_DIR), "/scrobblerqueue.txt", NULL);
 
-    if (!(fd = fopen(buf, "r")))
+    if (! g_file_test (path, G_FILE_TEST_EXISTS))
         return;
-    pdebug(fmt_vastr("Opening %s", buf), DEBUG);
-    fclose(fd);
+
+    AUDDBG ("Opening %s\n", path);
 
     gchar* cache;
     gchar** values;
     gchar** entry;
-    g_file_get_contents(buf, &cache, NULL, NULL);
+    g_file_get_contents (path, & cache, NULL, NULL);
     values = g_strsplit(cache, "\n", 0);
+    g_free (path);
 
     int x;
     for (x=0; values[x] && strlen(values[x]); x++) {
@@ -919,28 +969,26 @@ static void read_cache(void)
              * we'll continue to check the value anyway, for backwards-compatibility reasons.
              */
             if (!strncmp(entry[5], "L", 1)) {
-                Tuple *tuple = aud_tuple_new();
+                Tuple *tuple = tuple_new();
                 gchar* string_value;
                 string_value = xmms_urldecode_plain(artist);
-                aud_tuple_associate_string(tuple, FIELD_ARTIST, NULL, string_value);
+                tuple_set_str(tuple, FIELD_ARTIST, NULL, string_value);
                 g_free(string_value);
                 string_value = xmms_urldecode_plain(title);
-                aud_tuple_associate_string(tuple, FIELD_TITLE, NULL, string_value);
+                tuple_set_str(tuple, FIELD_TITLE, NULL, string_value);
                 g_free(string_value);
                 string_value = xmms_urldecode_plain(album);
-                aud_tuple_associate_string(tuple, FIELD_ALBUM, NULL, string_value);
+                tuple_set_str(tuple, FIELD_ALBUM, NULL, string_value);
                 g_free(string_value);
-                aud_tuple_associate_int(tuple, FIELD_TRACK_NUMBER, NULL, track);
+                tuple_set_int(tuple, FIELD_TRACK_NUMBER, NULL, track);
                 item = q_put(tuple, t, len);
 
-                aud_tuple_free(tuple);
+                tuple_unref(tuple);
 
-                pdebug(fmt_vastr("a[%d]=%s t[%d]=%s l[%d]=%d i[%d]=%d b[%d]=%s",
-                                 i, I_ARTIST(item),
-                                 i, I_TITLE(item),
-                                 i, I_LEN(item),
-                                 i, I_TIME(item),
-                                 i, I_ALBUM(item)), DEBUG);
+                if (item != NULL)
+                    AUDDBG ("a[%d]=%s t[%d]=%s l[%d]=%d i[%d]=%d b[%d]=%s\n", i,
+                     I_ARTIST (item), i, I_TITLE (item), i, I_LEN (item), i,
+                     I_TIME (item), i, I_ALBUM (item));
             }
 
             free(artist);
@@ -952,35 +1000,34 @@ static void read_cache(void)
     }
     g_strfreev(values);
     g_free(cache);
-    pdebug("Done loading cache.", DEBUG);
+    AUDDBG("Done loading cache.\n");
 }
 
 static void dump_queue(void)
 {
     FILE *fd;
     item_t *item;
-    char *home, buf[PATH_MAX];
-    gchar* config_datadir;
 
-    /*pdebug("Entering dump_queue();", DEBUG);*/
+    /*AUDDBG("Entering dump_queue();");*/
 
+    gchar * home;
     if (!(home = getenv("HOME")))
     {
-        pdebug("No HOME directory found. Cannot dump queue.", DEBUG);
+        AUDDBG("No HOME directory found. Cannot dump queue.\n");
         return;
     }
 
-    config_datadir = audacious_get_localdir();
-    g_snprintf(buf, sizeof(buf), "%s/scrobblerqueue.txt", config_datadir);
-    g_free(config_datadir);
+    gchar * path = g_strconcat (aud_get_path (AUD_PATH_USER_DIR), "/scrobblerqueue.txt", NULL);
 
-    if (!(fd = fopen(buf, "w")))
+    if (! (fd = fopen (path, "w")))
     {
-        pdebug(fmt_vastr("Failure opening %s", buf), DEBUG);
+        AUDDBG("Failure opening %s\n", path);
+        g_free (path);
         return;
     }
 
-    pdebug(fmt_vastr("Opening %s", buf), DEBUG);
+    AUDDBG("Opening %s\n", path);
+    g_free (path);
 
     q_peekall(1);
 
@@ -1023,7 +1070,7 @@ void sc_cleaner(void)
         free(sc_major_error);
     dump_queue();
     q_free();
-    pdebug("scrobbler shutting down", DEBUG);
+    AUDDBG("scrobbler shutting down\n");
 }
 
 static void sc_checkhandshake(void)
@@ -1050,8 +1097,8 @@ static void sc_checkhandshake(void)
                     (60 << (sc_hs_errors-5)) :
                     7200 );
             sc_hs_timeout = time(NULL) + wait;
-            pdebug(fmt_vastr("Error while handshaking. Retrying "
-                "after %d seconds.", wait), DEBUG);
+            AUDDBG("Error while handshaking. Retrying "
+                "after %d seconds.\n", wait);
         }
     }
 }
@@ -1060,7 +1107,7 @@ static void sc_checkhandshake(void)
 
 /* Called at session startup*/
 
-void sc_init(char *uname, char *pwd)
+void sc_init(char *uname, char *pwd, char *url)
 {
     sc_hs_status = sc_hs_timeout = sc_hs_errors = sc_submit_timeout =
         sc_srv_res_size = sc_giveup = sc_major_error_present =
@@ -1071,8 +1118,12 @@ void sc_init(char *uname, char *pwd)
         sc_challenge_hash = sc_major_error = NULL;
     sc_username = strdup(uname);
     sc_password = strdup(pwd);
+    if (url)
+        sc_hs_url = strdup(url);
+    else
+        sc_hs_url = strdup(LASTFM_HS_URL);
     read_cache();
-    pdebug("scrobbler starting up", DEBUG);
+    AUDDBG("scrobbler starting up\n");
 }
 
 void sc_addentry(GMutex *mutex, Tuple *tuple, int len)
