@@ -5,6 +5,8 @@
 //    Copyright (C) 2001  Florian Berger
 //    Email: florian.berger@jk.uni-linz.ac.at
 //
+//    Copyright (C) 2009-2011 John Lindgren
+//
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License Version 2 as
 //    published by the Free Software Foundation;
@@ -25,36 +27,48 @@
 #include "FB_logo.xpm"
 
 #include "config.h"
-#include <audacious/plugin.h>
-#include <audacious/i18n.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
+#include <audacious/gtk-compat.h>
+#include <audacious/i18n.h>
+#include <audacious/misc.h>
+#include <audacious/plugin.h>
+#include <libaudcore/audio.h>
+
 #define SNDSTRETCH_VERSION_STRING "0.7"
 
-#define SS sndstretch_var
-
-void sndstretch_init           (void);
+gboolean sndstretch_init (void);
 void sndstretch_about          (void);
 void sndstretch_config         (void);
-int sndstretch_mod_samples (gpointer *ptr, gint length, AFormat fmt, gint srate, gint nch);
 
-EffectPlugin sndstretch_ep = {
-	.description = "SndStretch",
+static void sndstretch_start (gint * channels, gint * rate);
+static void sndstretch_process (gfloat * * data, gint * samples);
+static void sndstretch_flush ();
+static void sndstretch_finish (gfloat * * data, gint * samples);
+static gint sndstretch_decoder_to_output_time (gint time);
+static gint sndstretch_output_to_decoder_time (gint time);
+
+AUD_EFFECT_PLUGIN
+(
+	.name = "SndStretch",
 	.init = sndstretch_init,
 	.about = sndstretch_about,
 	.configure = sndstretch_config,
-	.mod_samples = sndstretch_mod_samples,
-};
+    .start = sndstretch_start,
+    .process = sndstretch_process,
+    .flush = sndstretch_flush,
+    .finish = sndstretch_finish,
+    .decoder_to_output_time = sndstretch_decoder_to_output_time,
+    .output_to_decoder_time = sndstretch_output_to_decoder_time,
+    .preserves_format = TRUE,
+)
 
-EffectPlugin *sndstretch_eplist[] = { &sndstretch_ep, NULL };
-
-DECLARE_PLUGIN(sndstretch, NULL, NULL, NULL, NULL, sndstretch_eplist, NULL, NULL, NULL);
-
-static struct {
+struct sndstretch_settings
+{
 	int handle;    // file handle
 	int fragsize;
 	int chnr;
@@ -72,11 +86,10 @@ static struct {
 	double scale;
 	int short_overlap;
 	int volume_corr;
-	GtkObject * pitch_adj;
-	GtkObject * speed_adj;
-	GtkObject * scale_adj;
-} sndstretch_var;
+	GtkAdjustment * pitch_adj, * speed_adj, * scale_adj;
+};
 
+static struct sndstretch_settings SS;
 
 static const char sndstretch_title_text[] = "SndStretch xmms - " SNDSTRETCH_VERSION_STRING;
 
@@ -96,7 +109,7 @@ static const gchar sndstretch_GPL_text[] =
 	"GNU General Public License for more details.\n\n"
 	"You should have received a copy of the GNU General Public License "
 	"along with this program; if not, write to the Free Software "
-	"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, "
+	"Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, "
 	"USA.";
 
 GtkWidget * sndstretch_about_dialog  = NULL;
@@ -123,14 +136,6 @@ void sndstretch_about(void)
 	GtkWidget * text;
 	GtkTextBuffer * textbuffer;
 	GtkTextIter iter;
-
-	GdkPixmap * logopix;
-	GdkBitmap * logomask;
-	GtkWidget * logo;
-
-	GdkPixmap * FBlogopix;
-	GdkBitmap * FBlogomask;
-	GtkWidget * FBlogo;
 	GtkWidget * copyhbox, * copy_rbox, * copy_lbox;
 
 
@@ -140,21 +145,13 @@ void sndstretch_about(void)
 	sndstretch_about_dialog = gtk_dialog_new();
 	gtk_widget_show(sndstretch_about_dialog);
 
-	/* title logo */
-	logopix = gdk_pixmap_create_from_xpm_d(sndstretch_about_dialog->window, &logomask,
-										   NULL,
-										   (gchar **) sndstretch_xmms_logo_xpm);
-	logo = gtk_pixmap_new(logopix,logomask);
+	GtkWidget * logo = gtk_image_new_from_pixbuf (gdk_pixbuf_new_from_xpm_data
+	 ((const gchar * *) sndstretch_xmms_logo_xpm));
+	GtkWidget * FBlogo = gtk_image_new_from_pixbuf (gdk_pixbuf_new_from_xpm_data
+	 ((const gchar * *) FB_logo_xpm));
 
-	/* FB-logo */
-	FBlogopix = gdk_pixmap_create_from_xpm_d(sndstretch_about_dialog->window, &FBlogomask,
-											 NULL,
-											 (gchar **) FB_logo_xpm);
-	FBlogo = gtk_pixmap_new(FBlogopix,FBlogomask);
-
-
-	gtk_signal_connect(GTK_OBJECT(sndstretch_about_dialog), "destroy",
-					   GTK_SIGNAL_FUNC(sndstretch_about_destroy_cb), NULL);
+	g_signal_connect (sndstretch_about_dialog, "destroy", (GCallback)
+	 sndstretch_about_destroy_cb, NULL);
 	gtk_window_set_title(GTK_WINDOW(sndstretch_about_dialog), _("About SndStretch"));
 
 
@@ -172,8 +169,8 @@ void sndstretch_about(void)
 	gtk_box_pack_start(GTK_BOX(copyhbox), copy_rbox,    TRUE, TRUE,  5);
 
 	vbox = gtk_vbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sndstretch_about_dialog)->vbox), vbox,
-					   TRUE, TRUE, 5);
+	gtk_box_pack_start ((GtkBox *) gtk_dialog_get_content_area ((GtkDialog *)
+	 sndstretch_about_dialog), vbox, TRUE, TRUE, 5);
 
 	scrolltext = gtk_scrolled_window_new(NULL,NULL);
 	text = gtk_text_view_new();
@@ -196,14 +193,13 @@ void sndstretch_about(void)
 	gtk_box_pack_start(GTK_BOX(vbox), copyhbox, FALSE, TRUE, 5);
 	gtk_box_pack_start(GTK_BOX(vbox), scrolltext, TRUE, TRUE, 5);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
-	gtk_widget_set_usize(scrolltext, -1, 110);
 
 	button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sndstretch_about_dialog)->action_area),
-					   button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-					   GTK_SIGNAL_FUNC(sndstretch_about_ok_cb), NULL);
-	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+	gtk_box_pack_start ((GtkBox *) gtk_dialog_get_action_area ((GtkDialog *)
+	 sndstretch_about_dialog), button, FALSE, FALSE, 0);
+	g_signal_connect (button, "clicked", (GCallback) sndstretch_about_ok_cb,
+	 NULL);
+	gtk_widget_set_can_default (button, TRUE);
 	gtk_widget_grab_default(button);
 	gtk_widget_show(button);
 	gtk_widget_show_all(sndstretch_about_dialog);
@@ -213,30 +209,34 @@ void sndstretch_about(void)
 
 static void speed_change_cb(GtkAdjustment * adj, gpointer data)
 {
-	SS.speed = pow(2.0, GTK_ADJUSTMENT(adj)->value / (GTK_ADJUSTMENT(adj)->upper-10));
+	SS.speed = pow (2, gtk_adjustment_get_value (adj) /
+	 (gtk_adjustment_get_upper (adj) - 10));
 }
 
 static void pitch_change_cb(GtkAdjustment * adj, gpointer data)
 {
-	SS.pitch = pow(2.0, GTK_ADJUSTMENT(adj)->value / (GTK_ADJUSTMENT(adj)->upper-10));
-	gtk_adjustment_set_value(GTK_ADJUSTMENT(SS.scale_adj),
-							 (GTK_ADJUSTMENT(SS.scale_adj)->upper-10.0)*log(SS.pitch)/log(2.0));
+	SS.pitch = pow (2, gtk_adjustment_get_value (adj) /
+	 (gtk_adjustment_get_upper (adj) - 10));
+	gtk_adjustment_set_value (SS.scale_adj, (gtk_adjustment_get_upper
+	 (SS.scale_adj) - 10) * log (SS.pitch) / log (2));
 }
 
 static void scale_change_cb(GtkAdjustment * adj, gpointer data)
 {
 	double speed_eff;
 
-	SS.scale = pow(2.0, GTK_ADJUSTMENT(adj)->value / (GTK_ADJUSTMENT(adj)->upper-10));
+	SS.scale = pow (2, gtk_adjustment_get_value (adj) /
+	 (gtk_adjustment_get_upper (adj) - 10));
 	speed_eff= SS.speed/SS.pitch;
 	SS.pitch = SS.scale;
 	SS.speed = speed_eff*SS.scale;
 	if (SS.speed>2.0) SS.speed=2.0;
 	if (SS.speed<0.5) SS.speed=0.5;
-	gtk_adjustment_set_value(GTK_ADJUSTMENT(SS.speed_adj),
-							 (GTK_ADJUSTMENT(SS.speed_adj)->upper-10.0)*log(SS.speed)/log(2.0));
-	gtk_adjustment_set_value(GTK_ADJUSTMENT(SS.pitch_adj),
-							 (GTK_ADJUSTMENT(SS.pitch_adj)->upper-10.0)*log(SS.pitch)/log(2.0));
+
+	gtk_adjustment_set_value (SS.speed_adj, (gtk_adjustment_get_upper
+	 (SS.speed_adj) - 10) * log (SS.speed) / log (2));
+	gtk_adjustment_set_value (SS.pitch_adj, (gtk_adjustment_get_upper
+	 (SS.pitch_adj) - 10) * log (SS.pitch) / log (2));
 }
 
 static void overlap_toggle_cb(GtkToggleButton *butt, gpointer user_data)
@@ -256,15 +256,10 @@ static void sndstretch_config_logobutton_cb(GtkButton * button, gpointer data)
 
 static gint sndstretch_config_destroy_cb(GtkWidget * w, GdkEventAny * e, gpointer data)
 {
-	mcs_handle_t *db = aud_cfg_db_open();
-
-	aud_cfg_db_set_double(db, "sndstretch", "pitch", SS.pitch);
-	aud_cfg_db_set_double(db, "sndstretch", "speed", SS.speed);
-
-	aud_cfg_db_set_bool(db, "sndstretch", "short_overlap", SS.short_overlap);
-	aud_cfg_db_set_bool(db, "sndstretch", "volume_corr", SS.volume_corr);
-
-	aud_cfg_db_close(db);
+	aud_set_double ("sndstretch", "pitch", SS.pitch);
+	aud_set_double ("sndstretch", "speed", SS.speed);
+	aud_set_bool ("sndstretch", "short_overlap", SS.short_overlap);
+	aud_set_bool ("sndstretch", "volume_corr", SS.volume_corr);
 
 	gtk_widget_destroy(sndstretch_config_dialog);
 	sndstretch_config_dialog = NULL;
@@ -278,9 +273,6 @@ void sndstretch_config(void)
 	GtkWidget * speed_spin,  * pitch_spin,  * scale_spin;
 	GtkWidget * speed_hbox,  * pitch_hbox,  * scale_hbox,  * opt_hbox;
 	GtkWidget * speed_frame, * pitch_frame, * scale_frame, * opt_frame;
-	GdkPixmap * logopix;
-	GdkBitmap * logomask;
-	GtkWidget * logo;
 	GtkWidget * logohbox;
 	GtkWidget * logobutton;
 	GtkWidget * volume_toggle;
@@ -290,51 +282,44 @@ void sndstretch_config(void)
 		return;
 
 	sndstretch_config_dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+ 	gtk_window_set_type_hint(GTK_WINDOW(sndstretch_config_dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_widget_show(sndstretch_config_dialog);
 
-	logopix = gdk_pixmap_create_from_xpm_d(sndstretch_config_dialog->window, &logomask,
-										   NULL, (gchar **)sndstretch_xmms_logo_xpm);
-
-	logo = gtk_pixmap_new(logopix,logomask);
+	GtkWidget * logo = gtk_image_new_from_pixbuf (gdk_pixbuf_new_from_xpm_data
+	 ((const gchar * *) sndstretch_xmms_logo_xpm));
 
 	logobutton = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(logobutton), GTK_RELIEF_NONE);
 	gtk_container_add(GTK_CONTAINER(logobutton), logo);
-	gtk_signal_connect(GTK_OBJECT(logobutton), "clicked",
-					   GTK_SIGNAL_FUNC(sndstretch_config_logobutton_cb), NULL);
-	GTK_WIDGET_SET_FLAGS(logobutton, GTK_CAN_DEFAULT);
-	gtk_widget_grab_default(logobutton);
+	g_signal_connect (logobutton, "clicked", (GCallback)
+	 sndstretch_config_logobutton_cb, NULL);
+	gtk_widget_set_can_default (logobutton, TRUE);
 
 	logohbox = gtk_hbox_new(FALSE,0);  // to make it rightbound
 	gtk_box_pack_end(GTK_BOX(logohbox), logobutton, FALSE, TRUE, 4);
 
-	SS.speed_adj = gtk_adjustment_new( 100.0*log(SS.speed)/log(2.0),
-									   -100, 100+10, 2, 10, 10);
-	SS.pitch_adj = gtk_adjustment_new( 120.0*log(SS.pitch)/log(2.0),
-									   -120, 120+10, 2, 10, 10);
-	SS.scale_adj = gtk_adjustment_new( 100.0*log(SS.scale)/log(2.0),
-									   -100, 100+10, 2, 10, 10);
+	SS.speed_adj = (GtkAdjustment *) gtk_adjustment_new (100 * log (SS.speed) /
+	 log (2), -100, 100 + 10, 2, 10, 0);
+	SS.pitch_adj = (GtkAdjustment *) gtk_adjustment_new (120 * log (SS.pitch) /
+	 log (2), -120, 120 + 10, 2, 10, 0);
+	SS.scale_adj = (GtkAdjustment *) gtk_adjustment_new (100 * log (SS.scale) /
+	 log (2), -100, 100 + 10, 2, 10, 0);
 
 	volume_toggle  = gtk_check_button_new_with_label(_("Volume corr."));
 	overlap_toggle = gtk_check_button_new_with_label(_("Short Overlap"));
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(volume_toggle), SS.volume_corr );
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(overlap_toggle), SS.short_overlap );
 
-
-	gtk_signal_connect(GTK_OBJECT(SS.speed_adj), "value-changed",
-					   GTK_SIGNAL_FUNC(speed_change_cb), NULL);
-
-	gtk_signal_connect(GTK_OBJECT(SS.pitch_adj), "value-changed",
-					   GTK_SIGNAL_FUNC(pitch_change_cb), NULL);
-
-	gtk_signal_connect(GTK_OBJECT(SS.scale_adj), "value-changed",
-					   GTK_SIGNAL_FUNC(scale_change_cb), NULL);
-
-	gtk_signal_connect(GTK_OBJECT(volume_toggle), "toggled",
-					   GTK_SIGNAL_FUNC(volume_toggle_cb), NULL);
-
-	gtk_signal_connect(GTK_OBJECT(overlap_toggle), "toggled",
-					   GTK_SIGNAL_FUNC(overlap_toggle_cb), NULL);
+	g_signal_connect (SS.speed_adj, "value-changed", (GCallback)
+	 speed_change_cb, NULL);
+	g_signal_connect (SS.pitch_adj, "value-changed", (GCallback)
+	 pitch_change_cb, NULL);
+	g_signal_connect (SS.scale_adj, "value-changed", (GCallback)
+	 scale_change_cb, NULL);
+	g_signal_connect (volume_toggle, "toggled", (GCallback) volume_toggle_cb,
+	 NULL);
+	g_signal_connect (overlap_toggle, "toggled", (GCallback) overlap_toggle_cb,
+	 NULL);
 
 	speed_scale = gtk_hscale_new(GTK_ADJUSTMENT(SS.speed_adj));
 	pitch_scale = gtk_hscale_new(GTK_ADJUSTMENT(SS.pitch_adj));
@@ -346,9 +331,6 @@ void sndstretch_config(void)
 	speed_spin = gtk_spin_button_new(GTK_ADJUSTMENT(SS.speed_adj),1.0,2);
 	pitch_spin = gtk_spin_button_new(GTK_ADJUSTMENT(SS.pitch_adj),1.0,2);
 	scale_spin = gtk_spin_button_new(GTK_ADJUSTMENT(SS.scale_adj),1.0,2);
-	gtk_widget_set_usize (speed_spin,70,20);
-	gtk_widget_set_usize (pitch_spin,70,20);
-	gtk_widget_set_usize (scale_spin,70,20);
 	gtk_entry_set_max_length (GTK_ENTRY(pitch_spin),7);
 	gtk_entry_set_max_length (GTK_ENTRY(speed_spin),7);
 	gtk_entry_set_max_length (GTK_ENTRY(scale_spin),7);
@@ -390,22 +372,24 @@ void sndstretch_config(void)
 	gtk_box_pack_start(GTK_BOX(vbox), opt_frame,     FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), logohbox,      FALSE, TRUE, 0);
 
-	gtk_signal_connect(GTK_OBJECT(sndstretch_config_dialog), "destroy",
-					   GTK_SIGNAL_FUNC(sndstretch_config_destroy_cb), NULL);
+	g_signal_connect (sndstretch_config_dialog, "destroy", (GCallback)
+	 sndstretch_config_destroy_cb, NULL);
 	gtk_window_set_title(GTK_WINDOW(sndstretch_config_dialog), _("SndStretch - Configuration"));
 	gtk_container_add(GTK_CONTAINER(sndstretch_config_dialog), vbox);
 
-	gtk_widget_set_usize(sndstretch_config_dialog, -1, -1);
+	gtk_widget_grab_default(logobutton);
 	gtk_widget_show_all(sndstretch_config_dialog);
 }
 
+static const gchar * const sndstretch_defaults[] = {
+ "pitch", "1",
+ "speed", "1",
+ "short_overlap", "FALSE",
+ "volume_corr", "FALSE",
+ NULL};
 
-void sndstretch_init(void)
+gboolean sndstretch_init (void)
 {
-	mcs_handle_t *db;
-
-	db = aud_cfg_db_open();
-
 	SS.fragsize=0;
 	SS.chnr=2;
 	SS.paused=0;
@@ -417,38 +401,80 @@ void sndstretch_init(void)
 	SS.bpsec=176400;
 	SS.vol_r=50;
 	SS.vol_l=50;
-	SS.pitch=1.0;
-	SS.speed=1.0;
 	SS.scale=1.0;
 
-	gboolean b;
-	aud_cfg_db_get_double(db, "sndstretch", "pitch", &SS.pitch);
-	aud_cfg_db_get_double(db, "sndstretch", "speed", &SS.speed);
-	if (aud_cfg_db_get_bool(db, "sndstretch", "short_overlap", &b))
-		SS.short_overlap = b;
-	if (aud_cfg_db_get_bool(db, "sndstretch", "volume_corr", &b))
-		SS.volume_corr = b;
-	aud_cfg_db_close(db);
+	aud_config_set_defaults ("sndstretch", sndstretch_defaults);
+
+	SS.pitch = aud_get_double ("sndstretch", "pitch");
+	SS.speed = aud_get_double ("sndstretch", "speed");
+	SS.short_overlap = aud_get_bool ("sndstretch", "short_overlap");
+	SS.volume_corr = aud_get_bool ("sndstretch", "volume_corr");
+
+	return TRUE;
 }
 
-int sndstretch_mod_samples (gpointer *ptr, gint length, AFormat fmt, gint srate, gint nch)
+static gboolean initted = FALSE;
+static PitchSpeedJob job;
+static gint current_channels, current_rate;
+static struct sndstretch_settings current_settings;
+
+static void sndstretch_start (gint * channels, gint * rate)
 {
-	static short int * buff_o;
-	static int prod_size;
-	static PitchSpeedJob job;
-	static int init_job=1;
+    if (! initted)
+    {
+        InitPitchSpeedJob (& job);
+        initted = TRUE;
+    }
 
-	buff_o = realloc(buff_o, 65536);
+    current_channels = * channels;
+    current_rate = * rate;
+    memcpy (& current_settings, & SS, sizeof (struct sndstretch_settings));
+}
 
-	if (init_job) {
-		InitPitchSpeedJob(&job);
-		init_job = 0;
-	}
-	snd_pitch_speed_job(*ptr, nch, length/2, 0, SS.pitch, SS.speed,
-						(SS.short_overlap) ? 882 : 1764,
-						buff_o, &prod_size, &job, SS.volume_corr);
+/* FIXME: Find a stretch algorithm that uses floating point. */
+/* FIXME: The output buffer should be freed on plugin cleanup. */
+static void sndstretch_process (gfloat * * data, gint * samples)
+{
+    gint new_samples = (* samples) / current_settings.speed + 100;
+    gint16 * converted, * stretched;
+    static gfloat * reconverted = NULL;
 
-	*ptr = (gpointer) buff_o;
+    if (samples == 0)
+        return;
 
-	return prod_size*2;
+    converted = g_malloc (2 * (* samples));
+    audio_to_int (* data, converted, FMT_S16_NE, * samples);
+
+    stretched = g_malloc (2 * new_samples);
+    snd_pitch_speed_job (converted, current_channels, * samples, FALSE,
+     current_settings.pitch, current_settings.speed,
+     current_settings.short_overlap ? 882 : 1764, stretched, & new_samples,
+     & job, current_settings.volume_corr);
+    g_free (converted);
+
+    reconverted = g_realloc (reconverted, sizeof (gfloat) * new_samples);
+    audio_from_int (stretched, FMT_S16_NE, reconverted, new_samples);
+    g_free (stretched);
+
+    * data = reconverted;
+    * samples = new_samples;
+}
+
+static void sndstretch_flush ()
+{
+}
+
+static void sndstretch_finish (gfloat * * data, gint * samples)
+{
+    sndstretch_process (data, samples);
+}
+
+static gint sndstretch_decoder_to_output_time (gint time)
+{
+    return time / current_settings.speed;
+}
+
+static gint sndstretch_output_to_decoder_time (gint time)
+{
+    return time * current_settings.speed;
 }
